@@ -20,7 +20,7 @@
 #ifndef XAP_MacDIALOG_INSERT_SYMBOL_H
 #define XAP_MacDIALOG_INSERT_SYMBOL_H
 
-#include "xap_Dialog_Insert_Symbol.h"
+#include "xap_Dlg_Insert_Symbol.h"
 class XAP_MacFrame;
 
 /*****************************************************************/
@@ -31,8 +31,13 @@ public:
 	XAP_MacDialog_Insert_Symbol(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id);
 	virtual ~XAP_MacDialog_Insert_Symbol(void);
 
-	virtual void			runModal(XAP_Frame * pFrame);
-
+	virtual void			runModeless(XAP_Frame * pFrame);
+	virtual void			runModal(XAP_Frame * pFrame){};
+	virtual void			notifyActiveFrame(XAP_Frame *pFrame){};
+	virtual void			notifyCloseFrame(XAP_Frame *pFrame){};
+	virtual void			destroy(void){};
+	virtual void			activate(void){};
+ 
 	static XAP_Dialog *		static_constructor(XAP_DialogFactory *, XAP_Dialog_Id id);
 	
 protected:
