@@ -48,6 +48,9 @@
 
 /*****************************************************************/
 
+static UT_uint32 xap_UnixDlg_Insert_Symbol_first = 0;
+static UT_UCSChar m_CurrentSymbol;
+static UT_UCSChar m_PreviousSymbol;
 
 XAP_Dialog * XAP_UnixDialog_Insert_Symbol::static_constructor(XAP_DialogFactory * pFactory,
 															  XAP_Dialog_Id id)
@@ -72,7 +75,6 @@ XAP_UnixDialog_Insert_Symbol::XAP_UnixDialog_Insert_Symbol(XAP_DialogFactory * p
 
 	m_areaCurrentSym = NULL;
 	m_InsertS_Font_list = NULL;
-
 }
 
 XAP_UnixDialog_Insert_Symbol::~XAP_UnixDialog_Insert_Symbol(void)
@@ -84,15 +86,17 @@ XAP_UnixDialog_Insert_Symbol::~XAP_UnixDialog_Insert_Symbol(void)
 
 /*****************************************************************/
 
-static void s_ok_clicked(GtkWidget * widget, XAP_UnixDialog_Insert_Symbol * dlg)
+static gboolean s_ok_clicked(GtkWidget * widget, XAP_UnixDialog_Insert_Symbol * dlg)
 {
 	UT_ASSERT(widget && dlg);
 	dlg->event_OK();
+        return FALSE;
 }
-static void s_cancel_clicked(GtkWidget * widget, XAP_UnixDialog_Insert_Symbol * dlg)
+static gboolean s_cancel_clicked(GtkWidget * widget, XAP_UnixDialog_Insert_Symbol * dlg)
 {
 	UT_ASSERT(widget && dlg);
 	dlg->event_Cancel();
+        return FALSE;
 }
 
 
@@ -109,10 +113,11 @@ static void s_Symbolarea_exposed(GtkWidget * widget, GdkEvent * e, XAP_UnixDialo
 	dlg->Symbolarea_exposed();
 }
 
-static void s_SymbolMap_clicked(GtkWidget * widget, GdkEvent * e, XAP_UnixDialog_Insert_Symbol * dlg)
+static gboolean  s_SymbolMap_clicked(GtkWidget * widget, GdkEvent * e, XAP_UnixDialog_Insert_Symbol * dlg)
 {
 	UT_ASSERT(widget && dlg);
 	dlg->SymbolMap_clicked( e );
+        return FALSE; 
 }
 
 static void s_new_font(GtkWidget * widget, XAP_UnixDialog_Insert_Symbol * dlg)
@@ -135,14 +140,45 @@ static void s_delete_clicked(GtkWidget * /* widget */,
 	dlg->event_WindowDelete();
 }
 
+
 /*****************************************************************/
 
 void XAP_UnixDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 {
+}
+
+void XAP_UnixDialog_Insert_Symbol::activate(void)
+{
+        UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
+        gdk_window_raise(m_windowMain->window);
+}
+
+void   XAP_UnixDialog_Insert_Symbol::notifyActiveFrame(XAP_Frame *pFrame)
+{
+        UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
+}
+
+
+void XAP_UnixDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
+{
+	// First see if the dialog is already running
+	UT_sint32 sid =(UT_sint32)  getDialogId();
+	  
 	// Build the window's widgets and arrange them
 	GtkWidget * mainWindow = _constructWindow();
 	UT_ASSERT(mainWindow);
 
+	// Save dialog the ID number and pointer to the widget
+	m_pApp->rememberModelessId( sid,  (XAP_Dialog_Modeless *) m_pDialog);
+
+	// This magic command displays the frame that characters will be
+	// inserted into.
+	connectFocusModeless(GTK_WIDGET(mainWindow),m_pApp);
+
 	// To center the dialog, we need the frame of its parent.
 	XAP_UnixFrame * pUnixFrame = static_cast<XAP_UnixFrame *>(pFrame);
 	UT_ASSERT(pUnixFrame);
@@ -151,10 +187,10 @@ void XAP_UnixDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 	GtkWidget * parentWindow = pUnixFrame->getTopLevelWindow();
 	UT_ASSERT(parentWindow);
 	
-	// Center our new dialog in its parent and make it a transient
-	// so it won't get lost underneath
-    centerDialog(parentWindow, mainWindow);
-	gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
+	// Center our new dialog in its parent.
+	
+	// centerDialog(parentWindow, mainWindow);
+
 	gtk_widget_show (mainWindow);
 
 	// *** this is how we add the gc for symbol table ***
@@ -189,9 +225,9 @@ void XAP_UnixDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 	UT_ASSERT(iDrawSymbol);
 
 	// We use this code to insert the default font name into to static
-        // variable "m_Insert_Symbol_font" the first time this dialog is
-        // called. Afterwards it is just whatever was left from the last
-        // call.
+	// variable "m_Insert_Symbol_font" the first time this dialog is
+	// called. Afterwards it is just whatever was left from the last
+	// call.
 
 	if ( xap_UnixDlg_Insert_Symbol_first == 0)
 	{
@@ -205,12 +241,9 @@ void XAP_UnixDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 
 	gtk_widget_show(mainWindow);
 
-	// Make it modal, and stick it up top
-	gtk_grab_add(mainWindow);
-
         // Put the current font in the entry box
 	char* iSelectedFont = iDrawSymbol->getSelectedFont();
-        gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(m_fontcombo)->entry),
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(m_fontcombo)->entry),
 					   (gchar *)   iSelectedFont);
 
 	// Show the Previously selected symbol
@@ -218,17 +251,14 @@ void XAP_UnixDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 	m_PreviousSymbol = m_CurrentSymbol;
 	iDrawSymbol->drawarea(m_CurrentSymbol, m_PreviousSymbol);
 
-	// Run into the GTK event loop for this window.
-
-	gtk_main();
-
-	gtk_widget_destroy(mainWindow);
+	// return to ap_Editmethods and wait for something interesting
+	// to happen.
 }
 
 void XAP_UnixDialog_Insert_Symbol::event_OK(void)
 {
         m_Inserted_Symbol = m_CurrentSymbol;
-	_onInsertButton();
+       	_onInsertButton();
 }
 
 void XAP_UnixDialog_Insert_Symbol::event_Cancel(void)
@@ -243,9 +273,10 @@ void XAP_UnixDialog_Insert_Symbol::event_Cancel(void)
 			if(m_fontlist[i] != NULL)
 				g_free (m_fontlist[i]);
 		}
-
 		m_Insert_Symbol_no_fonts = 0;
-		gtk_main_quit();
+		modeless_cleanup();
+		gtk_widget_destroy(m_windowMain);
+		m_windowMain= NULL;
 	}
 }
 
@@ -289,6 +320,11 @@ void XAP_UnixDialog_Insert_Symbol::Key_Pressed(GdkEventKey * e)
 	case GDK_Right:
 		move = 1;
 		break;
+        case GDK_Return:
+	        gtk_signal_emit_stop_by_name((GTK_OBJECT(m_windowMain)),
+					     "key_press_event");
+			event_OK();
+		break;
 	}
 
 	if (move != 0)
@@ -340,13 +376,31 @@ void XAP_UnixDialog_Insert_Symbol::New_Font(void )
 	iDrawSymbol->drawarea(m_CurrentSymbol, m_PreviousSymbol);
 }
 
+
+void XAP_UnixDialog_Insert_Symbol::destroy(void)
+{
+
+	g_list_free( m_InsertS_Font_list);
+	for(UT_uint32 i = 0; i < m_Insert_Symbol_no_fonts; i++) g_free(m_fontlist[i]);
+        modeless_cleanup();
+
+	// Just nuke this dialog
+       
+	gtk_widget_destroy(m_windowMain);
+        m_windowMain = NULL;
+}
+
 void XAP_UnixDialog_Insert_Symbol::event_WindowDelete(void)
 {
 	m_answer = XAP_Dialog_Insert_Symbol::a_CANCEL;	
 	g_list_free( m_InsertS_Font_list);
+
 	for(UT_uint32 i = 0; i < m_Insert_Symbol_no_fonts; i++)
 		g_free(m_fontlist[i]);
-	gtk_main_quit();
+
+        modeless_cleanup();
+        gtk_widget_destroy(m_windowMain);
+        m_windowMain = NULL;
 }
 
 
@@ -354,77 +408,29 @@ void XAP_UnixDialog_Insert_Symbol::event_WindowDelete(void)
 
 GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 {
-	GtkWidget * windowInsertS;
-
 	GtkWidget * vboxInsertS;
 	GtkWidget * vhbox;
-
-	GtkWidget * fontcombo;
-	GtkWidget * SymbolMap;
-
-	GtkWidget * areaCurrentSym;
-
 	GtkWidget * hboxInsertS;
-	GtkWidget * buttonOK;
-	GtkWidget * buttonCancel;
-
-
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
 	XML_Char * tmp = NULL;
 
-	windowInsertS = gtk_window_new (GTK_WINDOW_DIALOG);
-	gtk_object_set_data (GTK_OBJECT (windowInsertS), "windowInsertS", windowInsertS);
-	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(XAP_STRING_ID_DLG_Insert_SymbolTitle));
-	gtk_window_set_title (GTK_WINDOW (windowInsertS), tmp);
-	FREEP(tmp);
-	gtk_widget_set_usize(windowInsertS, 610, 245);
+	m_windowMain = gtk_window_new (GTK_WINDOW_DIALOG);
+        
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
 
-	gtk_window_set_policy (GTK_WINDOW (windowInsertS), FALSE, FALSE, FALSE);
+	//	gtk_widget_set_usize(m_windowMain, 610, 245);
+	gtk_window_set_policy (GTK_WINDOW (m_windowMain), FALSE, FALSE, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (m_windowMain), 4);
 
 	// Now put in a Vbox to hold our 3 widgets (Font Selector, Symbol Table
 	// and OK -Selected Symbol- Cancel
-
-	vboxInsertS = gtk_vbox_new( FALSE, 1);
+	vboxInsertS = gtk_vbox_new (FALSE, 4);
+//	gtk_box_set_spacing (GTK_BOX (vboxInsertS), 4);
 	gtk_widget_show(vboxInsertS);
+
 	// Insert the vbox into the dialog window
-
-	gtk_container_add(GTK_CONTAINER(windowInsertS),vboxInsertS);
-
-	// Now Build the font combo box into the frame
-
-	/* First though we have to grab the fonts!
-
-	   This code is to suck all the available fonts and put them in a GList.
-	   This can then be displayed on a combo box at the top of the dialog.
-	   Code stolen from ap_UnixToolbar_FontCombo */
-
-	XAP_UnixApp * unixapp = static_cast<XAP_UnixApp *> (m_pApp);
-	UT_uint32 count = unixapp->getFontManager()->getCount();
-	XAP_UnixFont ** list = unixapp->getFontManager()->getAllFonts();
-
-	/* Now we remove all the duplicate name entries and create the Glist
-	   m_InsertS_Font_list. This will be used in the font selection combo
-	   box */
-
-	gchar currentfont[50] = "\0";
-	UT_uint32 j = 0;
-	m_InsertS_Font_list = NULL;
-
-	for (UT_uint32 i = 0; i < count; i++)
-	{
-		gchar * lgn  = (gchar *) list[i]->getName();
-		if(strstr(currentfont, lgn) == NULL)
-		{
-			strncpy(currentfont, lgn, 50);
-			m_fontlist[j] = g_strdup(currentfont);
-			m_InsertS_Font_list = g_list_prepend(m_InsertS_Font_list, m_fontlist[j++]);
-		}
-	}
-	m_Insert_Symbol_no_fonts = j;
-	DELETEP(list);
-  
-	m_InsertS_Font_list = g_list_reverse(m_InsertS_Font_list);
+	gtk_container_add(GTK_CONTAINER(m_windowMain), vboxInsertS);
 
 	vhbox = gtk_hbox_new(FALSE, 1);
 
@@ -433,9 +439,110 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 	gtk_box_pack_start(GTK_BOX(vboxInsertS), vhbox, TRUE, TRUE, 0);
 
 	// Finally construct the combo box
-	fontcombo = gtk_combo_new();
+	m_fontcombo = _createComboboxWithFonts ();
+	gtk_object_set_data (GTK_OBJECT(m_windowMain), "fontcombo", m_fontcombo);
+
+	// Now put the font combo box at the top of the dialog 
+	gtk_box_pack_start(GTK_BOX(vhbox), m_fontcombo, TRUE, FALSE, 0);
+
+	// Now the Symbol Map. 
+	// TODO: 32 * x (19) = 608, 7 * y (21) = 147  FIXME!
+	m_SymbolMap = _previewNew (608, 147);
+	gtk_object_set_data (GTK_OBJECT (m_windowMain), "SymbolMap", m_SymbolMap);
+	gtk_box_pack_start(GTK_BOX(vboxInsertS), m_SymbolMap, FALSE, FALSE, 0);
+	
+	// Now make a Hbox to hold  OK, Current Selection and Cancel
+	hboxInsertS = gtk_hbox_new (FALSE, 1);
+
+	// Insert the hbox into the dialog window
+	gtk_object_set_data (GTK_OBJECT (m_windowMain), "hboxInsertS", hboxInsertS);
+	gtk_widget_show (hboxInsertS);
+	gtk_box_pack_start (GTK_BOX (vboxInsertS), hboxInsertS, TRUE, TRUE, 0);
+
+	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(XAP_STRING_ID_DLG_Insert));
+	m_buttonOK = gtk_button_new_with_label (tmp);
+	FREEP(tmp);
+
+	gtk_object_set_data (GTK_OBJECT (m_windowMain), "buttonOK", m_buttonOK);
+	gtk_widget_show (m_buttonOK);
+	gtk_box_pack_start(GTK_BOX(hboxInsertS), m_buttonOK, TRUE, FALSE, 4);
+	GTK_WIDGET_SET_FLAGS (m_buttonOK, GTK_CAN_DEFAULT);
+
+	m_areaCurrentSym = _previewNew (60, 45);
+	gtk_object_set_data (GTK_OBJECT (m_windowMain), 
+						 "areaCurrentSym", m_areaCurrentSym);
+	gtk_box_pack_start(GTK_BOX(hboxInsertS), m_areaCurrentSym, TRUE, FALSE, 0);
+
+	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(XAP_STRING_ID_DLG_Close));
+	m_buttonCancel = gtk_button_new_with_label (tmp);
+	FREEP(tmp);
+
+	gtk_object_set_data (GTK_OBJECT (m_windowMain), "buttonCancel", m_buttonCancel);
+	gtk_widget_show (m_buttonCancel);
+	gtk_box_pack_start(GTK_BOX(hboxInsertS), m_buttonCancel, TRUE, FALSE, 4);
+	GTK_WIDGET_SET_FLAGS (m_buttonCancel, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (hboxInsertS);
+
+	_connectSignals ();
+
+	gtk_widget_grab_focus (m_buttonOK);
+	gtk_widget_grab_default (m_buttonOK);
+	
+	return m_windowMain;
+}
+
+GtkWidget *XAP_UnixDialog_Insert_Symbol::_previewNew (int w, int h)
+{
+	GtkWidget *pre = gtk_drawing_area_new ();
+	gtk_widget_show (pre);
+	gtk_widget_set_usize (pre, w, h);
+	
+	// Enable button press events
+	gtk_widget_add_events(pre, GDK_BUTTON_PRESS_MASK);
+	return pre;
+}
+	
+/*
+  This code is to suck all the available fonts and put them in a GList.
+  This can then be displayed on a combo box at the top of the dialog.
+  Code stolen from ap_UnixToolbar_FontCombo */
+/* Now we remove all the duplicate name entries and create the Glist
+   glFonts. This will be used in the font selection combo
+   box */
+
+GList *XAP_UnixDialog_Insert_Symbol::_getGlistFonts (void)
+{	  
+	XAP_UnixApp * unixapp = static_cast<XAP_UnixApp *> (m_pApp);
+	UT_uint32 count = unixapp->getFontManager()->getCount();
+	XAP_UnixFont ** list = unixapp->getFontManager()->getAllFonts();
+	GList *glFonts = NULL;
+	gchar currentfont[50] = "\0";
+	UT_uint32 j = 0;
+	
+	for (UT_uint32 i = 0; i < count; i++)
+	{
+		gchar * lgn  = (gchar *) list[i]->getName();
+		if(strstr(currentfont, lgn) == NULL)
+		{
+			strncpy(currentfont, lgn, 50);
+			m_fontlist[j] = g_strdup(currentfont);
+			glFonts = g_list_prepend(glFonts, m_fontlist[j++]);
+		}
+	}
+
+	m_Insert_Symbol_no_fonts = j;
+	DELETEP(list);
+
+	return g_list_reverse(glFonts);
+}
+
+GtkWidget *XAP_UnixDialog_Insert_Symbol::_createComboboxWithFonts (void)
+{
+	GtkWidget *fontcombo = gtk_combo_new();
+
+	m_InsertS_Font_list = _getGlistFonts ();
  
-	gtk_object_set_data (GTK_OBJECT(windowInsertS), "fontcombo", fontcombo);
 	gtk_widget_set_usize(fontcombo, 200, 25);
 	gtk_widget_show(fontcombo);
 	gtk_combo_set_value_in_list(GTK_COMBO(fontcombo), TRUE, TRUE);
@@ -449,95 +556,30 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 	// Turn off keyboard entry in the font selection box
 	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(fontcombo)->entry),FALSE);
 
-	// Now put the font combo box at the top of the dialog 
-	gtk_box_pack_start(GTK_BOX(vhbox), fontcombo, TRUE, FALSE, 0);
-
-	// Now the Symbol Map. 
-
-	// *** Code Stolen from the preview widget ***
-	{
-		SymbolMap = gtk_drawing_area_new ();
-		gtk_object_set_data (GTK_OBJECT (windowInsertS), "SymbolMap", SymbolMap);
-		gtk_widget_show (SymbolMap);
-
-		// TODO: 32 * x (19) = 608, 7 * y (21) = 147  FIXME!
-		gtk_widget_set_usize (SymbolMap, 608, 147);
-		// Put this in the dialog under the font selection widget
-
-		gtk_box_pack_start(GTK_BOX(vboxInsertS), SymbolMap, FALSE, FALSE, 0);
-
-		// Enable button press events
-		gtk_widget_add_events(SymbolMap, GDK_BUTTON_PRESS_MASK);
-   	}
-	
-	// Now make a Hbox to hold  OK, Current Selection and Cancel
-	hboxInsertS = gtk_hbox_new (FALSE, 1);
-
-	// Insert the hbox into the dialog window
-	gtk_object_set_data (GTK_OBJECT (windowInsertS), "hboxInsertS", hboxInsertS);
-	gtk_widget_show (hboxInsertS);
-	gtk_box_pack_start (GTK_BOX (vboxInsertS), hboxInsertS, TRUE, TRUE, 0);
-
-	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(XAP_STRING_ID_DLG_Insert));
-	buttonOK = gtk_button_new_with_label (tmp);
-        FREEP(tmp);
-
-	gtk_object_set_data (GTK_OBJECT (windowInsertS), "buttonOK", buttonOK);
-	gtk_widget_show (buttonOK);
-	gtk_box_pack_start(GTK_BOX(hboxInsertS), buttonOK, TRUE, FALSE, 4);
-	GTK_WIDGET_SET_FLAGS (buttonOK, GTK_CAN_DEFAULT);
-
-	// *** Code Stolen from the preview widget again! ***
-	{
-		areaCurrentSym = gtk_drawing_area_new ();
-		gtk_object_set_data (GTK_OBJECT (windowInsertS), 
-							 "areaCurrentSym", areaCurrentSym);
-		gtk_widget_show (areaCurrentSym);
-		gtk_widget_set_usize (areaCurrentSym, 60,45);
-		gtk_box_pack_start(GTK_BOX(hboxInsertS), areaCurrentSym, TRUE, FALSE, 0);
-   	}
-
-	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(XAP_STRING_ID_DLG_Close));
-	buttonCancel = gtk_button_new_with_label (tmp);
-        FREEP(tmp);
-
-	gtk_object_set_data (GTK_OBJECT (windowInsertS), "buttonCancel", buttonCancel);
-	gtk_widget_show (buttonCancel);
-	gtk_box_pack_start(GTK_BOX(hboxInsertS), buttonCancel, TRUE, FALSE, 4);
-	GTK_WIDGET_SET_FLAGS (buttonCancel, GTK_CAN_DEFAULT);
-
-	gtk_widget_show (hboxInsertS);
-
-	// Update member variables with the important widgets that
-	// might need to be queried or altered later.
-	m_windowMain = windowInsertS;
-
-	m_buttonOK = buttonOK;
-	m_buttonCancel = buttonCancel;
-
-	m_SymbolMap = 	SymbolMap;
-	m_fontcombo = fontcombo;
-	m_areaCurrentSym = areaCurrentSym;
+	return fontcombo;
+}
 
+void XAP_UnixDialog_Insert_Symbol::_connectSignals (void)
+{
 	// Now connect the signals
-	gtk_signal_connect(GTK_OBJECT(buttonOK),
+	gtk_signal_connect(GTK_OBJECT(m_buttonOK),
 					   "clicked",
 					   GTK_SIGNAL_FUNC(s_ok_clicked),
 					   (gpointer) this);
 	
-	gtk_signal_connect(GTK_OBJECT(buttonCancel),
+	gtk_signal_connect(GTK_OBJECT(m_buttonCancel),
 					   "clicked",
 					   GTK_SIGNAL_FUNC(s_cancel_clicked),
 					   (gpointer) this);
 
 	// The event to choose the Symbol!
-	gtk_signal_connect(GTK_OBJECT(SymbolMap),
+	gtk_signal_connect(GTK_OBJECT(m_SymbolMap),
 					   "button_press_event",
 				       GTK_SIGNAL_FUNC(s_SymbolMap_clicked),
 					   (gpointer) this);
 
 	// Look for keys pressed
-	gtk_signal_connect(GTK_OBJECT(windowInsertS),
+	gtk_signal_connect(GTK_OBJECT(m_windowMain),
 					   "key_press_event",
 					   GTK_SIGNAL_FUNC(s_keypressed),
 					   (gpointer) this);
@@ -545,8 +587,7 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 
 	// Look for "changed" signal on the entry part of the combo box.
 	// Code stolen from ev_UnixGnomeToolbar.cpp
-
-	GtkEntry * blah = GTK_ENTRY(GTK_COMBO(fontcombo)->entry);
+	GtkEntry * blah = GTK_ENTRY(GTK_COMBO(m_fontcombo)->entry);
 	GtkEditable * yuck = GTK_EDITABLE(blah);
 	gtk_signal_connect(GTK_OBJECT(&yuck->widget),
 					   "changed",
@@ -554,31 +595,22 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 					   (gpointer) this);
 
 	// the catch-alls
-	gtk_signal_connect_after(GTK_OBJECT(windowInsertS),
+	gtk_signal_connect_after(GTK_OBJECT(m_windowMain),
 							 "delete_event",
 							 GTK_SIGNAL_FUNC(s_delete_clicked),
 							 (gpointer) this);
 
-	gtk_signal_connect_after(GTK_OBJECT(windowInsertS),
+	gtk_signal_connect_after(GTK_OBJECT(m_windowMain),
 							 "destroy",NULL, NULL);
 	
-	// the expose event of the SymbolMap
-	gtk_signal_connect(GTK_OBJECT(SymbolMap),
+	// the expose event of the m_SymbolMap
+	gtk_signal_connect(GTK_OBJECT(m_SymbolMap),
 					   "expose_event",
 					   GTK_SIGNAL_FUNC(s_sym_SymbolMap_exposed),
 					   (gpointer) this);
 
-	gtk_signal_connect(GTK_OBJECT(areaCurrentSym),
+	gtk_signal_connect(GTK_OBJECT(m_areaCurrentSym),
 					   "expose_event",
 					   GTK_SIGNAL_FUNC(s_Symbolarea_exposed),
 					   (gpointer) this);
-
-	gtk_widget_grab_focus (buttonOK);
-	gtk_widget_grab_default (buttonOK);
-// We have to wait a little...
-//	gtk_widget_show(windowInsertS);
-	
-	return windowInsertS;
 }
-
-
