@@ -38,22 +38,9 @@
 
 static void _notifyError_OKOnly(XAP_Frame * pFrame, const char * message)
 {
-	XAP_DialogFactory * pDialogFactory
-		= (XAP_DialogFactory *)(pFrame->getDialogFactory());
-
-	XAP_Dialog_MessageBox * pDialog
-		= (XAP_Dialog_MessageBox *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_MESSAGE_BOX));
-	UT_ASSERT(pDialog);
-
-	pDialog->setMessage(message);
-	pDialog->setButtons(XAP_Dialog_MessageBox::b_O);
-	pDialog->setDefaultAnswer(XAP_Dialog_MessageBox::a_OK);
-
-	pDialog->runModal(pFrame);
-
-//	XAP_Dialog_MessageBox::tAnswer ans = pDialog->getAnswer();
-
-	pDialogFactory->releaseDialog(pDialog);
+	pFrame->showMessageBox(message,
+								XAP_Dialog_MessageBox::b_O,
+								XAP_Dialog_MessageBox::a_OK);
 }
 
 /*****************************************************************/
@@ -231,6 +218,7 @@ void XAP_UnixDialog_Print::_raisePrintDialog(XAP_Frame * pFrame)
 	
 	// Create window
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	connectFocus(GTK_WIDGET(window),pFrame);
 	gtk_signal_connect_after(GTK_OBJECT(window),
 							  "destroy",
 							  NULL,
@@ -507,6 +495,7 @@ void XAP_UnixDialog_Print::_raisePrintDialog(XAP_Frame * pFrame)
 	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(parent));
 	
 	gtk_widget_show (window);
+	gtk_grab_add(GTK_WIDGET(window));
 
 	gtk_main();
 
