@@ -520,7 +520,7 @@ UT_Bool PS_Graphics::_startPage(const char * szPageLabel, UT_uint32 pageNumber,
 	char buf[1024];
 	sprintf(buf,"%d",pageNumber);
 
-	const char * argv[2] = { szPageLabel, buf };
+	const char * argv[2] = { buf, buf };
 	m_ps->formatComment("Page",argv,2);
 
 	// TODO add other page-header comments here
@@ -618,7 +618,7 @@ void PS_Graphics::_emit_IncludeResource(void)
 		if(!unixfont->openPFA())
 		{
 			char message[1024];
-			g_snprintf(message, 2048,
+			g_snprintf(message, sizeof (message),
 					   "Font data file [%s] cannot be opened for reading!\n"
 					   "Did it disappear on us?  AbiWord can't print without\n"
 					   "this file; your PostScript might be missing this resource.",
@@ -738,18 +738,18 @@ void PS_Graphics::_emit_SetColor(void)
 	switch(m_cs)
 	{
 	case GR_Graphics::GR_COLORSPACE_COLOR:
-		sprintf(buf,"%.8f %.8f %.8f setrgbcolor\n",					// TODO do we need to deal with locale for PS
-				((float) m_currentColor.m_red / (float) 255.0),
-				((float) m_currentColor.m_grn / (float) 255.0),
-				((float) m_currentColor.m_blu / (float) 255.0));
+		sprintf(buf,"0.%.0f 0.%.0f 0.%.0f setrgbcolor\n",					// Write the radix character as hard wired periods in PS files, LC_NUMERIC can make sprintf choose non period values for it.
+				((float) m_currentColor.m_red / (float) 255.0)*100000000.0,
+				((float) m_currentColor.m_grn / (float) 255.0)*100000000.0,
+				((float) m_currentColor.m_blu / (float) 255.0)*100000000.0);
 		break;
 	case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
 		newclr = (unsigned char) (( (float) m_currentColor.m_red +
 									(float) m_currentColor.m_grn +
 									(float) m_currentColor.m_blu ) /
 								  (float) 3.0);
-		sprintf(buf,"%.8f setgray\n",								// TODO do we need to deal with locale for PS
-				(float) newclr / (float) 255.0);
+		sprintf(buf,"0.%.0f setgray\n",								// Write the radix character as hard wired periods in PS files, LC_NUMERIC can make sprintf choose non period values for it.
+				(float) newclr / (float) 255.0) * 10000000.0;
 		break;
 	case GR_Graphics::GR_COLORSPACE_BW:
 		// Black & White is a special case of the Gray color space where
