@@ -199,7 +199,7 @@ void _updateDirAndSpace(char* pszPath)
 
 static void _png_read(png_structp png_ptr, png_bytep data, png_size_t length)
 {
-	struct _bb* p = (struct _bb*) png_ptr->io_ptr;
+	struct _bb* p = (struct _bb*) png_get_io_ptr(png_ptr);
 
 	memcpy(data, p->pBytes + p->iCurPos, length);
 	p->iCurPos += length;
