@@ -51,6 +51,8 @@
 #include "xap_Clipboard.h"
 #include "ut_png.h"
 #include "fg_Graphic.h"
+#include "ap_Prefs.h"
+#include "ap_Prefs_SchemeIds.h"
 
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
@@ -59,7 +61,10 @@
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
-#define IsZeroLengthTextRun(p)		(((p)->getLength()==0) && ((p)->getType()==FPRUN_TEXT))
+inline UT_Bool IsZeroLengthTextRun(const fp_Run* p)
+{
+	return p->getLength() == 0 && p->getType() == FPRUN_TEXT;
+}
 
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
@@ -107,7 +112,6 @@ fl_BlockLayout::fl_BlockLayout(PL_StruxDocHandle sdh,
 	{
 		m_pNext->m_pPrev = this;
 	}
-
 }
 
 fl_TabStop::fl_TabStop()
@@ -195,7 +199,7 @@ void fl_BlockLayout::_lookupProperties(void)
 	}
 	
 	GR_Graphics* pG = m_pLayout->getGraphics();
-	
+
 	m_iTopMargin = pG->convertDimension(getProperty("margin-top"));
 	m_iTopMarginLayoutUnits = UT_convertToLayoutUnits(getProperty("margin-top"));
 	m_iBottomMargin = pG->convertDimension(getProperty("margin-bottom"));
@@ -279,9 +283,6 @@ void fl_BlockLayout::_lookupProperties(void)
 			{
 				switch (p1[1])
 				{
-				case 'L':
-					iType = FL_TAB_LEFT;
-					break;
 				case 'R':
 					iType = FL_TAB_RIGHT;
 					break;
@@ -294,6 +295,7 @@ void fl_BlockLayout::_lookupProperties(void)
 				case 'B':
 					iType = FL_TAB_BAR;
 					break;
+				case 'L':	// fall through
 				default:
 					iType = FL_TAB_LEFT;
 					break;
@@ -303,13 +305,10 @@ void fl_BlockLayout::_lookupProperties(void)
 			char pszPosition[32];
 			UT_uint32 iPosLen = p1 - pStart;
 		
-			UT_ASSERT(iPosLen < 32);
+			UT_ASSERT(iPosLen < sizeof pszPosition);
 
-			for (i=0; i<iPosLen; i++)
-			{
-				pszPosition[i] = pStart[i];
-			}
-			pszPosition[i] = 0;
+			memcpy(pszPosition, pStart, iPosLen);
+            pszPosition[iPosLen] = 0;
 
 			iPosition = pG->convertDimension(pszPosition);
 
@@ -381,7 +380,7 @@ void fl_BlockLayout::_lookupProperties(void)
 			m_dLineSpacing = pG->convertDimension(pTmp);
 			m_dLineSpacingLayoutUnits = UT_convertToLayoutUnits(pTmp);
 		}
-		else if(UT_hasDimensionComponent(pszSpacing))
+		else if (UT_hasDimensionComponent(pszSpacing))
 		{
 			m_eSpacingPolicy = spacing_EXACT;
 			m_dLineSpacing = pG->convertDimension(pszSpacing);
@@ -631,7 +630,11 @@ UT_Bool fl_BlockLayout::truncateLayout(fp_Run* pTruncRun)
 void fl_BlockLayout::checkForBeginOnForcedBreak(void)
 {
 	fp_Line* pFirstLine = getFirstLine();
+	UT_ASSERT(pFirstLine);
+
 	fp_Run* pFirstRun = pFirstLine->getLastRun();
+	UT_ASSERT(pFirstRun);
+
 	if (pFirstRun->canContainPoint())
 	{
 		return;
@@ -642,10 +645,13 @@ void fl_BlockLayout::checkForBeginOnForcedBreak(void)
 	*/
 
 	GR_Graphics* pG = m_pLayout->getGraphics();
+	UT_ASSERT(pG);
+
 	fp_TextRun* pNewRun = new fp_TextRun(this, pG, m_pFirstRun->getBlockOffset(), 0);
-	pNewRun->setPrev(NULL);
-	pNewRun->setNext(m_pFirstRun);
-	m_pFirstRun->setPrev(pNewRun);
+	// TODO: Check for out-of-memory. Only assert for now
+	UT_ASSERT(pNewRun);
+
+	m_pFirstRun->insertIntoRunListBeforeThis(*pNewRun);
 	pFirstLine->insertRun(pNewRun);
 	m_pFirstRun = pNewRun;
 
@@ -655,7 +661,11 @@ void fl_BlockLayout::checkForBeginOnForcedBreak(void)
 void fl_BlockLayout::checkForEndOnForcedBreak(void)
 {
 	fp_Line* pLastLine = getLastLine();
+	UT_ASSERT(pLastLine);
+	
 	fp_Run* pLastRun = pLastLine->getLastRun();
+	UT_ASSERT(pLastRun);
+
 	if (pLastRun->canContainPoint())
 	{
 		return;
@@ -665,14 +675,13 @@ void fl_BlockLayout::checkForEndOnForcedBreak(void)
 	  We add a zero-length text run on a line by itself.
 	*/
 
-	fp_Line* pNewLine;
-	
 	GR_Graphics* pG = m_pLayout->getGraphics();
 	fp_TextRun* pNewRun = new fp_TextRun(this, pG, pLastRun->getBlockOffset() + pLastRun->getLength(), 0);
-	pNewRun->setPrev(pLastRun);
-	pLastRun->setNext(pNewRun);
 
-	pNewLine = getNewLine();
+	pLastRun->insertIntoRunListAfterThis(*pNewRun);
+
+	fp_Line* pNewLine = getNewLine();
+	UT_ASSERT(pNewLine);
 	UT_ASSERT(pNewLine == m_pLastLine);
 
 	// the line just contains the empty run
@@ -714,21 +723,123 @@ void fl_BlockLayout::_insertFakeTextRun(void)
 	m_pFirstLine->layout();
 }
 
-int fl_BlockLayout::format()
+// Split the line the Run resides on, ensuring that the resulting two lines
+// are both valid in the sense of being able to contain the point.
+void fl_BlockLayout::_breakLineAfterRun(fp_Run* pRun)
 {
-	if (m_bFixCharWidths)
+	// When loading a document, there may not have been created
+	// lines yet. Get a first one created and hope for the best...
+	if (!m_pFirstLine)
+		_stuffAllRunsOnALine();
+
+	// First make sure the old line can contain the point
+	fp_Line* pLine = pRun->getLine();
+	// (This is a sloppy check - may have to loop between the two Runs
+	//  and check that at least one Run can contain the point.)
+	if (pLine->getFirstRun() == pRun)
 	{
-		fp_Run* pRun = m_pFirstRun;
-		while (pRun)
-		{
-			pRun->recalcWidth();
-			
-			pRun = pRun->getNext();
-		}
+		// Add a new zero-length Run at the start of the old
+		// line.
+		GR_Graphics* pG = m_pLayout->getGraphics();
+		fp_TextRun* pNewRun = new fp_TextRun(this, pG, pRun->getBlockOffset(), 0);
+		pRun->insertIntoRunListBeforeThis(*pNewRun);
+		pLine->insertRun(pNewRun);
+		// Update FirstRun if necessary
+		if (pLine->isFirstLineInBlock())
+		    m_pFirstRun = pNewRun;
+	}
+
+	// Create the new line
+	fp_Line* pNewLine = new fp_Line();
+	UT_ASSERT(pNewLine);
+	// Insert it after the current line
+	pNewLine->setPrev(pLine);
+	pNewLine->setNext(pLine->getNext());
+	pLine->setNext(pNewLine);
+	// Update LastLine if necessary
+	if (m_pLastLine == pLine)
+	    m_pLastLine = pNewLine;
+	// Set the block
+	pNewLine->setBlock(this);
+	// Add the line to the container
+	pLine->getContainer()->insertLineAfter(pNewLine, pLine);
+
+	// Now add Runs following pRun on the same line to the new
+	// line. Keep track of whether any Runs that can contain Point
+	// are added.
+	UT_Bool bCanContaintPoint = UT_FALSE;
+	fp_Run* pCurrentRun = pRun->getNext();
+	while (pCurrentRun && pCurrentRun->getLine() == pLine)
+	{
+		pLine->removeRun(pCurrentRun, UT_TRUE);
+		pNewLine->addRun(pCurrentRun);
+		if (pCurrentRun->canContainPoint())
+			bCanContaintPoint = UT_TRUE;
+		pCurrentRun = pCurrentRun->getNext();
 	}
 	
+	if (!bCanContaintPoint)
+	{
+		// No Runs that can contain Point were added to the
+		// new line. Add a new zero-length Run at the start of
+		// the new line.
+		GR_Graphics* pG = m_pLayout->getGraphics();
+		fp_TextRun* pNewRun = new fp_TextRun(this, pG, pRun->getBlockOffset() + pRun->getLength(), 0);
+		pRun->insertIntoRunListAfterThis(*pNewRun);
+		pNewLine->insertRun(pNewRun);
+	}
+
+	// Update the layout information in the lines.
+	pLine->layout();
+	pNewLine->layout();
+}
+
+UT_Bool fl_BlockLayout::_validateBlockForPoint(void)
+{
+	// check that all lines in the block have at least one run
+	// which can contain the insertion point.
+
+	fp_Line* pLine = getFirstLine();
+	while (pLine)
+	{
+		UT_Bool bCanContainPoint = UT_FALSE;
+		
+		fp_Run* pRun = pLine->getFirstRun();
+		while (pRun)
+		{
+			if (pRun->canContainPoint())
+			{
+				bCanContainPoint = UT_TRUE;
+				break;
+			}
+			pRun = pRun->getNext();
+		}
+
+		if (!bCanContainPoint)
+		    return UT_FALSE;
+
+		pLine = pLine->getNext();
+	}
+
+	return UT_TRUE;
+}
+
+
+int fl_BlockLayout::format()
+{
 	if (m_pFirstRun)
 	{
+		if (m_bFixCharWidths)
+		{
+			fp_Run* pRun = m_pFirstRun;
+			while (pRun)
+			{
+				pRun->recalcWidth();
+				
+				pRun = pRun->getNext();
+			}
+		}
+
 		if (!m_pFirstLine)
 			_stuffAllRunsOnALine();
 
@@ -769,6 +880,7 @@ void fl_BlockLayout::redrawUpdate()
 fp_Line* fl_BlockLayout::getNewLine(void)
 {
 	fp_Line* pLine = new fp_Line();
+	// TODO: Handle out-of-memory
 	UT_ASSERT(pLine);
 
 	pLine->setBlock(this);
@@ -870,116 +982,212 @@ UT_Bool	fl_BlockLayout::getBlockBuf(UT_GrowBuf * pgb) const
 	return m_pDoc->getBlockBuf(m_sdh, pgb);
 }
 
-fp_Run* fl_BlockLayout::findPointCoords(PT_DocPosition iPos, UT_Bool bEOL, UT_sint32& x, UT_sint32& y, UT_sint32& height)
+fp_Run* fl_BlockLayout::findPointCoords(PT_DocPosition	iPos,
+                                        UT_Bool			bEOL,
+                                        UT_sint32&		x,
+                                        UT_sint32&		y,
+                                        UT_sint32&		height)
 {
-	// find the run which has this position inside it.
+	// Compute insertion point coordinates and size
 
-	PT_DocPosition dPos = getPosition();
-	
-	UT_ASSERT(iPos >= dPos);
-	UT_uint32 iRelOffset = iPos - dPos;
-	if(iRelOffset < 0) iRelOffset = 0;
 	if (!m_pFirstLine || !m_pFirstRun)
 	{
 		// when we have no formatting information, can't find anything
 		return NULL;
 	}
 
+	// find the run which has this offset inside it.
+	PT_DocPosition dPos = getPosition();
+	UT_ASSERT(iPos >= dPos);
+	const UT_uint32 iRelOffset = iPos - dPos;
+
+	// By default, the Run just before the one we find is the one we
+	// want the coords from. This is because insertion is done with
+	// the properties of the Run before the point.
+	// In some situations, we need to override that and use the coords
+	// of the found Run - this flag tells us when to do what.
+	UT_Bool bCoordOfPrevRun = UT_TRUE;
+
+	// Some of the special cases below fix up pRun/bCoordOfPrevRun
+	// so we can use the first exit point. We could just as well
+	// fiddle bEOL in those cases, but using this variable makes
+	// the intention more clear (IMO).
+	UT_Bool bUseFirstExit = UT_FALSE;
+
+	// Find first Run past (or at) the requested offset. By scanning
+	// in this manner, we do a mimimum of computation to find the
+	// approximate location.
 	fp_Run* pRun = m_pFirstRun;
-	while (pRun)
-	{
-		UT_uint32 iWhere = pRun->containsOffset(iRelOffset);
-		if (FP_RUN_JUSTAFTER == iWhere)
-		{
-		       if(pRun->getNext())
-		       {
-			    if(pRun->getNext()->containsOffset(iRelOffset) == FP_RUN_INSIDE)
-			    {
-			          pRun->findPointCoords(iRelOffset, x, y, height);
-				  return pRun->getNext();
-			    }
-		       }
-		}
-		if (FP_RUN_INSIDE == iWhere)
-		{
-		  pRun->findPointCoords(iRelOffset, x, y, height);	
-		return pRun;
-		}
-		else if (bEOL && (FP_RUN_JUSTAFTER == iWhere))
-		{
-			fp_Run* pNext = pRun->getNext();
-			fp_Line* pNextLine = NULL;
-
-			if (pNext)
-				pNextLine = pNext->getLine();
-
-			if (pNextLine != pRun->getLine())
-			{
-				pRun->findPointCoords(iRelOffset, x, y, height);
-				return pRun;
-			}
-		}
-		
+	while (pRun && pRun->getBlockOffset() < iRelOffset)
 		pRun = pRun->getNext();
-	}
 
-	pRun = m_pFirstRun;
-	while (pRun)
-	{
-		UT_uint32 iWhere = pRun->containsOffset(iRelOffset);
-		if ((FP_RUN_JUSTAFTER == iWhere))
-		{
-		    fp_Run* nextRun = pRun->getNext();
-		    if(nextRun)	
-		    {
-		          nextRun->lookupProperties();
-		          nextRun->findPointCoords(iRelOffset, x, y, height);
-		    }
-		    else
-		    {
-		          pRun->findPointCoords(iRelOffset, x, y, height);
-		    }
-		return pRun;
-		}
-
-		if (!pRun->getNext())
-		{
-			// this is the last run, we're not going to get another chance, so try harder
-			if (iRelOffset > (pRun->getBlockOffset() + pRun->getLength()))
-			{
-				pRun->findPointCoords(iRelOffset, x, y, height);
-				return pRun;
-			}
-		}
+	// Now scan farther if necessary - the block may contain Runs
+	// with zero length. This is only a problem when empty Runs
+	// appear for no good reason (i.e., an empty Run on an empty
+	// line should be OK).
+	while (pRun && pRun->getBlockOffset() + pRun->getLength() < iRelOffset)
 		pRun = pRun->getNext();
-	}
 
-	if (iRelOffset < m_pFirstRun->getBlockOffset())
+	// We may have scanned past the last Run in the block. Back up.
+	if (!pRun)
 	{
-		m_pFirstRun->findPointCoords(iRelOffset, x, y, height);
-		return m_pFirstRun;
-	}
-
-	pRun = m_pFirstRun;
-	while (pRun)
-	{
-		if (pRun->canContainPoint())
-		{
-			if(!pRun->getNext())
-			  {
-			    pRun->findPointCoords(iRelOffset, x, y, height);
-			  }
-			else
-			  {
-			    pRun->getNext()->findPointCoords(iRelOffset, x, y, height);
-			  }
-			return pRun;
-		}
-		pRun = pRun->getNext();
+		pRun = getLastLine()->getLastRun();
+		bCoordOfPrevRun = UT_FALSE;
 	}
 	
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	return NULL;
+	// Step one back if if previous Run holds the offset (the
+	// above loops scan past what we're looking for since it's
+	// faster).
+	fp_Run* pPrevRun = pRun->getPrev();
+	if (pPrevRun && 
+	    pPrevRun->getBlockOffset() + pPrevRun->getLength() > iRelOffset)
+	{
+	    pRun = pPrevRun;
+	    bCoordOfPrevRun = UT_FALSE;
+	}
+
+	// Since the requested offset may be a page break (or similar
+	// Runs) which cannot contain the point, now work backwards
+	// while looking for a Run which can contain the point.
+	while (pRun && !pRun->canContainPoint()) 
+	{
+	    pRun = pRun->getPrev();
+	    bCoordOfPrevRun = UT_FALSE;
+	}
+
+	// Assert if there have been no Runs which can hold the point
+	// between the beginning of the block and the requested
+	// offset.
+	UT_ASSERT(NULL != pRun);
+
+	// This covers a special case (I) when bEOL.  Consider this
+	// line (| is the right margin, E end of document):
+	// 
+	// 1:  abcdefgh|
+	// 2:  iE
+	// 
+	// When EOL position for display line 1 is requested, it's
+	// done with either the offset of h or i (fall through to code
+	// below first exit point). EOL position for display line 2 is
+	// requested with offset of E (matches third sub-expresion).
+	// (This check is rather non-intuitive - step through the code
+	// for the different permutations to see why it's correct).
+	if (bEOL && pRun->getBlockOffset() < iRelOffset &&
+	    pRun->getBlockOffset() + pRun->getLength() >= iRelOffset)
+	{
+	    bCoordOfPrevRun = UT_FALSE;
+	    bUseFirstExit = UT_TRUE;
+	}
+
+	// If not bEOL, we're done: either we have actually found the
+	// Run containing the offset, or we have found the first
+	// suitable Run before the requested offset.
+	//
+	// This is the exit point most calls will use (being the first
+	// exit point, we should be OK performance wise).
+	if (bUseFirstExit || !bEOL)
+	{
+	    if (bCoordOfPrevRun && pRun->letPointPass())
+	    {
+		// This looks a little weird. What it does is first
+		// try to go one Run back only, if allowed.
+		// If that fails, use the original Run.
+		pPrevRun = pRun->getPrev();
+		if (!pPrevRun 
+		    || !pPrevRun->letPointPass()
+		    || !pPrevRun->canContainPoint())
+		    pPrevRun = pRun;
+		else
+		{
+		    // If the code gets one Run back, keep going back
+		    // until finding a Run that is valid for point
+		    // coordinate calculations.
+		    while (pPrevRun && 
+			   !pPrevRun->letPointPass() 
+			   || !pPrevRun->canContainPoint())
+			pPrevRun = pPrevRun->getPrev();
+
+		    // If this fails, go with the original Run.
+		    if (!pPrevRun)
+			pPrevRun = pRun;
+		}
+
+
+		// One final check: only allow the point to move to a
+		// different line if bEOL.
+		if (!bEOL && pRun->getLine() != pPrevRun->getLine())
+		    pPrevRun = pRun;
+
+		pPrevRun->findPointCoords(iRelOffset, x, y, height);
+	    } else {
+		pRun->findPointCoords(iRelOffset, x, y, height);
+	    }
+        
+	    return pRun;
+	}
+
+	// Runs with layout information (page/column break) are not
+	// visible (or rather, cannot contain the point). They look
+	// like this (P is a page break, E is end of document):
+	//
+	// 1:  abcdefghP
+	// <page/column alignment>
+	// 2:  E
+	//
+	// When we have to find EOL position for display line 2, the
+	// arguments (the offset) is the same as in case (I) above
+	// (i.e., the argument is the offset of P). Thus this special
+	// check.
+	pPrevRun = pRun->getPrev();
+	if (!pPrevRun || !pPrevRun->letPointPass())
+	{
+	    pRun->findPointCoords(iRelOffset, x, y, height);	
+	    return pRun;
+	}
+
+	// Now search for the Run at the end of the line.  For a
+	// soft-broken block (soft-break due to margin constraints),
+	// this may be on the previous display line. It may also be
+	// pRun (if the offset was past the last Run of this display
+	// line). Consider this line (| is the right margin, N the
+	// line break or paragraph end):
+	// 
+	// 1:  abcdefgh|
+	// 2:  ijklN
+	// 
+	// For normal cursor movement (bEOL=false), IP (*) will move
+	// from *h to *i (or vice versa), skipping the h* position.
+	// When bEOL=true (user presses End key, or selects EOL with
+	// mouse) IP on display line 1 will be at h*, even though the
+	// requested offset is actually that of i.
+	// 
+	while (pPrevRun && !pPrevRun->canContainPoint())
+	    pPrevRun = pPrevRun->getPrev();
+
+	// If we went past the head of the list, it means that the
+	// originally found Run is the only one on this display line.
+	if (!pPrevRun)
+	{
+	    pRun->findPointCoords(iRelOffset, x, y, height);	
+	    return pRun;
+	}
+
+
+	// If the Runs are on the same line, assume pRun to be farther
+	// right than pPrevRun.
+	if (pPrevRun->getLine() == pRun->getLine())
+	{
+	    pRun->findPointCoords(iRelOffset, x, y, height);	
+	    return pRun;
+	}
+
+	// Only case left is that of a soft-broken line.
+
+	// Always return position _and_ Run of the previous line. Old
+	// implementation returned pRun, but this will cause the
+	// cursor to wander if End is pressed multiple times.
+	pPrevRun->findPointCoords(iRelOffset, x, y, height);
+	return pPrevRun; 
 }
 
 fp_Line* fl_BlockLayout::findPrevLineInDocument(fp_Line* pLine)
@@ -1375,6 +1583,13 @@ void fl_BlockLayout::_mergeSquiggles(UT_uint32 iOffset, fl_BlockLayout* pPrevBL)
 
 void fl_BlockLayout::_recalcPendingWord(UT_uint32 iOffset, UT_sint32 chg)
 {
+	// If spell-check-as-you-type is off, we don't want a pending word at all
+	if (!m_pLayout->getAutoSpellCheck())
+	{
+		m_pLayout->setPendingWord(NULL, NULL);
+		return;
+	}
+	
 	UT_ASSERT(chg);
 
 	// on entrance, the block is already changed & any pending word is junk
@@ -1564,6 +1779,7 @@ void fl_BlockLayout::checkSpelling(void)
 
 	if (bUpdateScreen && pView)
 	{
+	        pView->_eraseInsertionPoint();
 		pView->updateScreen();
 		pView->_drawInsertionPoint();
 	}
@@ -1870,7 +2086,11 @@ UT_Bool	fl_BlockLayout::_doInsertForcedLineBreakRun(PT_BlockOffset blockOffset)
 	fp_Run* pNewRun = new fp_ForcedLineBreakRun(this, m_pLayout->getGraphics(), blockOffset, 1);
 	UT_ASSERT(pNewRun);	// TODO check for outofmem
 
-	return _doInsertRun(pNewRun);
+	UT_Bool bResult = _doInsertRun(pNewRun);
+	if (bResult)
+	    _breakLineAfterRun(pNewRun);
+
+	return bResult;
 }
 
 UT_Bool	fl_BlockLayout::_doInsertForcedPageBreakRun(PT_BlockOffset blockOffset)
@@ -1878,7 +2098,11 @@ UT_Bool	fl_BlockLayout::_doInsertForcedPageBreakRun(PT_BlockOffset blockOffset)
 	fp_Run* pNewRun = new fp_ForcedPageBreakRun(this, m_pLayout->getGraphics(), blockOffset, 1);
 	UT_ASSERT(pNewRun);	// TODO check for outofmem
 
-	return _doInsertRun(pNewRun);
+	UT_Bool bResult = _doInsertRun(pNewRun);
+	if (bResult)
+	    _breakLineAfterRun(pNewRun);
+
+	return bResult;
 }
 
 UT_Bool	fl_BlockLayout::_doInsertForcedColumnBreakRun(PT_BlockOffset blockOffset)
@@ -1886,7 +2110,11 @@ UT_Bool	fl_BlockLayout::_doInsertForcedColumnBreakRun(PT_BlockOffset blockOffset
 	fp_Run* pNewRun = new fp_ForcedColumnBreakRun(this, m_pLayout->getGraphics(), blockOffset, 1);
 	UT_ASSERT(pNewRun);	// TODO check for outofmem
 
-	return _doInsertRun(pNewRun);
+	UT_Bool bResult = _doInsertRun(pNewRun);
+	if (bResult)
+	    _breakLineAfterRun(pNewRun);
+
+	return bResult;
 }
 
 UT_Bool	fl_BlockLayout::_doInsertTabRun(PT_BlockOffset blockOffset)
@@ -1977,14 +2205,7 @@ UT_Bool	fl_BlockLayout::_doInsertRun(fp_Run* pNewRun)
 			// the insert is right before this run.
 			pRun->setBlockOffset(iRunBlockOffset + len);
 
-			pNewRun->setPrev(pRun->getPrev());
-			pNewRun->setNext(pRun);
-			if (pRun->getPrev())
-			{
-				pRun->getPrev()->setNext(pNewRun);
-			}
-
-			pRun->setPrev(pNewRun);
+			pRun->insertIntoRunListBeforeThis(*pNewRun);
 
 			if (m_pFirstRun == pRun)
 			{
@@ -2028,19 +2249,7 @@ UT_Bool	fl_BlockLayout::_doInsertRun(fp_Run* pNewRun)
 			// the insert is right before this run.
 			pRun->setBlockOffset(iRunBlockOffset + len);
 
-			pNewRun->setPrev(pRun->getPrev());
-			pNewRun->setNext(pRun);
-			if (pRun->getPrev())
-			{
-				pRun->getPrev()->setNext(pNewRun);
-			}
-
-			pRun->setPrev(pNewRun);
-
-			if (m_pFirstRun == pRun)
-			{
-				m_pFirstRun = pNewRun;
-			}
+			pRun->insertIntoRunListBeforeThis(*pNewRun);
 
 			pRun->getLine()->insertRunBefore(pNewRun, pRun);
 			
@@ -2065,8 +2274,7 @@ UT_Bool	fl_BlockLayout::_doInsertRun(fp_Run* pNewRun)
 
 		if (pLastRun)
 		{
-			pLastRun->setNext(pNewRun);
-			pNewRun->setPrev(pLastRun);
+			pLastRun->insertIntoRunListAfterThis(*pNewRun);
 		}
 		else
 		{
@@ -2136,11 +2344,11 @@ UT_Bool fl_BlockLayout::doclistener_insertSpan(const PX_ChangeRecord_Span * pcrs
 				break;
 				
 			case UCS_LF:
-				_doInsertForcedLineBreakRun(blockOffset + i);
+				_doInsertForcedLineBreakRun(i + blockOffset);
 				break;
 				
 			case UCS_TAB:
-				_doInsertTabRun(blockOffset + i);
+				_doInsertTabRun(i + blockOffset);
 				break;
 				
 			default:
@@ -2165,9 +2373,11 @@ UT_Bool fl_BlockLayout::doclistener_insertSpan(const PX_ChangeRecord_Span * pcrs
 	{
 		_doInsertTextSpan(blockOffset + iNormalBase, i - iNormalBase);
 	}
-	
+
 	setNeedsReformat();
 
+	UT_ASSERT(_validateBlockForPoint());
+
 	FV_View* pView = m_pLayout->getView();
 	if (pView)
 	{
@@ -2175,7 +2385,8 @@ UT_Bool fl_BlockLayout::doclistener_insertSpan(const PX_ChangeRecord_Span * pcrs
 		pView->notifyListeners(AV_CHG_FMTCHAR); // TODO verify that this is necessary.
 	}
 
-	_insertSquiggles(blockOffset, len);
+	if (m_pLayout->getAutoSpellCheck())
+		_insertSquiggles(blockOffset, len);
 
 	return UT_TRUE;
 }
@@ -2271,27 +2482,25 @@ UT_Bool fl_BlockLayout::_delete(PT_BlockOffset blockOffset, UT_uint32 len)
 
 			if ((pRun->getLength() == 0) && (pRun->getType() != FPRUN_FMTMARK))
 			{
+				// delete this run
 				fp_Line* pLine = pRun->getLine();
 				UT_ASSERT(pLine);
 
 				pLine->removeRun(pRun, UT_TRUE);
 
-				if (pRun->getNext())
-				{
-					pRun->getNext()->setPrev(pRun->getPrev());
-				}
-
-				if (pRun->getPrev())
-				{
-					pRun->getPrev()->setNext(pRun->getNext());
-				}
-
 				if (m_pFirstRun == pRun)
 				{
 					m_pFirstRun = pRun->getNext();
 				}
 
+				pRun->unlinkFromRunList();
+
 				delete pRun;
+
+				if (!m_pFirstRun)
+				{
+					_insertFakeTextRun();
+				}
 			}
 		}
 
@@ -2324,7 +2533,8 @@ UT_Bool fl_BlockLayout::doclistener_deleteSpan(const PX_ChangeRecord_Span * pcrs
  		pView->_setPoint(pcrs->getPosition());
 	}
 
-	_deleteSquiggles(blockOffset, len);
+	if (m_pLayout->getAutoSpellCheck())
+		_deleteSquiggles(blockOffset, len);
 
 	return UT_TRUE;
 }
@@ -2383,7 +2593,7 @@ UT_Bool fl_BlockLayout::doclistener_changeSpan(const PX_ChangeRecord_SpanChange
 	return UT_TRUE;
 }
 
-UT_Bool fl_BlockLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
+UT_Bool fl_BlockLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux* pcrx)
 {
 	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_DeleteStrux);
 	UT_ASSERT(pcrx->getStruxType()==PTX_Block);
@@ -2475,15 +2685,31 @@ UT_Bool fl_BlockLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pc
 				delete pNuke;
 			}
 
+#if 0		      
+			//BUG: Code from Mike that crashes hard
+
+			UT_ASSERT(pRun);
+
 			m_pFirstRun = pRun;
 
 			// then link what's left
+			m_pFirstRun->insertIntoRunListAfterThis(*pLastRun);
+
+
+#else
+			//BUG: Quick and Dirty Replacement Code
+
+			m_pFirstRun = pRun;
+
+			//then link what's left
 			pLastRun->setNext(m_pFirstRun);
 
-			if (m_pFirstRun)
+			if(m_pFirstRun)
 			{
 				m_pFirstRun->setPrev(pLastRun);
 			}
+#endif
+			
 		}
 		else
 		{
@@ -2541,29 +2767,18 @@ UT_Bool fl_BlockLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pc
 		// move all squiggles to previous block
 		_mergeSquiggles(offset, pPrevBL);
 
-		// in case we've never checked this one
-		m_pLayout->dequeueBlock(this);
-
 		// update the display
 //		pPrevBL->_lookupProperties();	// TODO: this may be needed
 		pPrevBL->setNeedsReformat();
-
-		FV_View* pView = pSL->getDocLayout()->getView();
-		if (pView)
-		{
-			pView->_setPoint(pcrx->getPosition());
-		}
 	}
-	else
+
+	// in case we've never checked this one
+	m_pLayout->dequeueBlock(this);
+
+	FV_View* pView = pSL->getDocLayout()->getView();
+	if (pView)
 	{
-		// in case we've never checked this one
-		m_pLayout->dequeueBlock(this);
-		
-		FV_View* pView = pSL->getDocLayout()->getView();
-		if (pView)
-		{
-			pView->_setPoint(pcrx->getPosition());
-		}
+		pView->_setPoint(pcrx->getPosition());
 	}
 
 	delete this;			// TODO whoa!  this construct is VERY dangerous.
@@ -2626,10 +2841,10 @@ UT_Bool fl_BlockLayout::doclistener_insertFirstBlock(const PX_ChangeRecord_Strux
 	//  Exchange handles with the piece table
 	PL_StruxFmtHandle sfhNew = (PL_StruxFmtHandle)this;
 	pfnBindHandles(sdh,lid,sfhNew);
-	
+
 	_insertFakeTextRun();
 	setNeedsReformat();
-	
+
 	FV_View* pView = m_pLayout->getView();
 	if (pView)
 	{
@@ -2648,7 +2863,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 {
 	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_InsertStrux);
 	UT_ASSERT(pcrx->getStruxType()==PTX_Block);
-					
+
 	fl_SectionLayout* pSL = m_pSectionLayout;
 	UT_ASSERT(pSL);
 	fl_BlockLayout*	pNewBL = pSL->insertBlock(sdh, this, pcrx->getIndexAP());
@@ -2683,11 +2898,11 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 	// figure out where the breakpoint is
 	PT_BlockOffset blockOffset = (pcrx->getPosition() - getPosition());
 
-	fp_Run * pFirstNewRun = NULL;
-	fp_Run * pLastRun = NULL;
-        fp_Run * pRun;
+	fp_Run* pFirstNewRun = NULL;
+	fp_Run* pLastRun = NULL;
+	fp_Run* pRun;
 	for (pRun=m_pFirstRun; (pRun && !pFirstNewRun); pLastRun=pRun, pRun=pRun->getNext())
-	{			
+	{
 		switch (pRun->containsOffset(blockOffset))
 		{
 		case FP_RUN_INSIDE:
@@ -2705,10 +2920,6 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 				pFirstNewRun = pRun;
 			}
 			break;
-			
-		case FP_RUN_JUSTAFTER:				// no split needed
-			pFirstNewRun = pRun->getNext();
-			break;
 
 		case FP_RUN_NOT:
 			break;
@@ -2726,7 +2937,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 		// to the right of the FmtMark, so we take the next one.
 		pFirstNewRun = pFirstNewRun->getNext();
 	}
-	
+
 	if (pFirstNewRun && pFirstNewRun->getPrev())
 	{
 		// break doubly-linked list of runs into two distinct lists
@@ -2734,7 +2945,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 		pFirstNewRun->getPrev()->setNext(NULL);
 		pFirstNewRun->setPrev(NULL);
 	}
-	
+
 	// pFirstNew can be NULL at this point.  it means that
 	// the entire set of runs in this block must remain
 	// with this block -- and the newly created block will
@@ -2775,8 +2986,9 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 		coalesceRuns();
 	else
 		_insertFakeTextRun();
+
 	setNeedsReformat();
-	
+
 	// throw all the runs onto one jumbo line in the new block
 	pNewBL->_stuffAllRunsOnALine();
 	if (pNewBL->m_pFirstRun)
@@ -2784,7 +2996,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 	else
 		pNewBL->_insertFakeTextRun();
 	pNewBL->setNeedsReformat();
-	
+
 	FV_View* pView = m_pLayout->getView();
 	if (pView)
 	{
@@ -2792,7 +3004,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 	}
 
 #ifdef FASTSQUIGGLE
-	if (m_vecSquiggles.getItemCount() > 0)
+	if (m_pLayout->getAutoSpellCheck() && m_vecSquiggles.getItemCount() > 0)
 	{
 		// we have squiggles, so move them 
 		_breakSquiggles(blockOffset, pNewBL);
@@ -2805,7 +3017,7 @@ UT_Bool fl_BlockLayout::doclistener_insertBlock(const PX_ChangeRecord_Strux * pc
 		m_pLayout->queueBlockForSpell(this);
 		m_pLayout->queueBlockForSpell(pNewBL);
 	}
-	
+
 	return UT_TRUE;
 }
 
@@ -2822,14 +3034,17 @@ UT_Bool fl_BlockLayout::doclistener_insertSection(const PX_ChangeRecord_Strux *
 	// that this insertion point is at the end of the block (and that
 	// another block follows).  this is because because a section
 	// cannot contain content.
-	
-	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_InsertStrux);
-	UT_ASSERT(pcrx->getStruxType()==PTX_Section);
+
+	UT_ASSERT(pcrx);
+	UT_ASSERT(pfnBindHandles);
+	UT_ASSERT(pcrx->getType() == PX_ChangeRecord::PXT_InsertStrux);
+	UT_ASSERT(pcrx->getStruxType() == PTX_Section);
 
 	UT_ASSERT(m_pSectionLayout->getType() == FL_SECTION_DOC);
 	fl_DocSectionLayout* pDSL = (fl_DocSectionLayout*) m_pSectionLayout;
 	
 	fl_DocSectionLayout* pSL = new fl_DocSectionLayout(m_pLayout, sdh, pcrx->getIndexAP());
+
 	if (!pSL)
 	{
 		UT_DEBUGMSG(("no memory for SectionLayout"));
@@ -2901,7 +3116,7 @@ void fl_BlockLayout::findSquigglesForRun(fp_Run* pRun)
 			{
 				iStart = pPOB->iOffset;
 			}
-		
+
 			if ((pPOB->iOffset + pPOB->iLength) >= (runBlockOffset + runLength))
 			{
 				iLen = runLength + runBlockOffset - iStart;
@@ -2963,7 +3178,7 @@ UT_Bool fl_BlockLayout::doclistener_insertObject(const PX_ChangeRecord_Object *
 		blockOffset = pcro->getBlockOffset();
 
 		FG_Graphic* pFG = FG_Graphic::createFromChangeRecord(this, pcro);
-		if(pFG == NULL)
+		if (pFG == NULL)
 			return UT_FALSE;
 
 		_doInsertImageRun(blockOffset, pFG);
@@ -2993,7 +3208,8 @@ UT_Bool fl_BlockLayout::doclistener_insertObject(const PX_ChangeRecord_Object *
 		pView->_setPoint(pcro->getPosition() + 1);
 	}
 
-	_insertSquiggles(blockOffset, 1);	// TODO: are objects always one wide?
+	if (m_pLayout->getAutoSpellCheck())
+		_insertSquiggles(blockOffset, 1);	// TODO: are objects always one wide?
 
 	return UT_TRUE;
 }
@@ -3034,7 +3250,8 @@ UT_Bool fl_BlockLayout::doclistener_deleteObject(const PX_ChangeRecord_Object *
 		pView->_setPoint(pcro->getPosition());
 	}
 
-	_deleteSquiggles(blockOffset, 1);	// TODO: are objects always one wide?
+	if (m_pLayout->getAutoSpellCheck())
+		_deleteSquiggles(blockOffset, 1);	// TODO: are objects always one wide?
 
 	return UT_TRUE;
 }
@@ -3097,9 +3314,9 @@ UT_Bool fl_BlockLayout::recalculateFields(void)
 	{
 		if (pRun->getType() == FPRUN_FIELD)
 		{
-			fp_FieldRun* pFieldRun = (fp_FieldRun*) pRun;
+			fp_FieldRun* pFieldRun = static_cast<fp_FieldRun*>(pRun);
 
-			UT_Bool bSizeChanged = pFieldRun->calculateValue();
+			const UT_Bool bSizeChanged = pFieldRun->calculateValue();
 
 			bResult = bResult || bSizeChanged;
 		}
@@ -3112,11 +3329,14 @@ UT_Bool fl_BlockLayout::recalculateFields(void)
 
 UT_Bool	fl_BlockLayout::findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX, UT_sint32& iPosition, unsigned char& iType)
 {
+	UT_ASSERT(iStartX >= 0);
+
 	UT_uint32 iCountTabs = m_vecTabs.getItemCount();
 	UT_uint32 i;
 	for (i=0; i<iCountTabs; i++)
 	{
 		fl_TabStop* pTab = (fl_TabStop*) m_vecTabs.getNthItem(i);
+		UT_ASSERT(pTab);
 
 		if (pTab->iPosition > iMaxX)
 		{
@@ -3144,6 +3364,9 @@ UT_Bool	fl_BlockLayout::findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX, UT_s
 	}
 	
 	UT_ASSERT(m_iDefaultTabInterval > 0);
+
+#if 0
+	// TMN: Original brute-force
 	UT_sint32 iPos = 0;
 	for (;;)
 	{
@@ -3160,10 +3383,24 @@ UT_Bool	fl_BlockLayout::findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX, UT_s
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 
 	return UT_FALSE;
+#else
+	// mathematical approach
+	const UT_sint32 iPos = (iStartX / m_iDefaultTabInterval + 1) *
+							m_iDefaultTabInterval;
+	iPosition = iPos;
+	iType = FL_TAB_LEFT;
+
+	UT_ASSERT(iPos > iStartX);
+
+	return UT_TRUE;
+#endif
+
 }
 
 UT_Bool	fl_BlockLayout::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32 iMaxX, UT_sint32& iPosition, unsigned char& iType)
 {
+	UT_ASSERT(iStartX >= 0);
+
 	UT_uint32 iCountTabs = m_vecTabs.getItemCount();
 	UT_uint32 i;
 	for (i=0; i<iCountTabs; i++)
@@ -3183,7 +3420,7 @@ UT_Bool	fl_BlockLayout::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint3
 			return UT_TRUE;
 		}
 	}
-	
+
 	// now, handle the default tabs
 
 	UT_sint32 iMin = m_iLeftMarginLayoutUnits;
@@ -3194,8 +3431,11 @@ UT_Bool	fl_BlockLayout::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint3
 		iType = FL_TAB_LEFT;
 		return UT_TRUE;
 	}
-	
+
 	UT_ASSERT(m_iDefaultTabIntervalLayoutUnits > 0);
+
+#if 0
+	// TMN: Original brute-force
 	UT_sint32 iPos = 0;
 	for (;;)
 	{
@@ -3212,6 +3452,18 @@ UT_Bool	fl_BlockLayout::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint3
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 
 	return UT_FALSE;
+#else
+	// mathematical approach
+	const UT_sint32 iPos = (iStartX / m_iDefaultTabIntervalLayoutUnits + 1) *
+							m_iDefaultTabIntervalLayoutUnits;
+	UT_ASSERT(iPos > iStartX);
+
+	iPosition = iPos;
+	iType = FL_TAB_LEFT;
+
+	return UT_TRUE;
+#endif
+
 }
 
 UT_Bool fl_BlockLayout::s_EnumTabStops(void * myThis, UT_uint32 k, UT_sint32 & iPosition, unsigned char & iType, UT_uint32 & iOffset)
@@ -3223,6 +3475,7 @@ UT_Bool fl_BlockLayout::s_EnumTabStops(void * myThis, UT_uint32 k, UT_sint32 & i
 	UT_uint32 iCountTabs = pBL->m_vecTabs.getItemCount();
 	if (k >= iCountTabs)
 		return UT_FALSE;
+
 	fl_TabStop * pTab = (fl_TabStop *)pBL->m_vecTabs.getNthItem(k);
 
 	iPosition = pTab->iPosition;
@@ -3361,15 +3614,11 @@ UT_Bool fl_BlockLayout::_deleteFmtMark(PT_BlockOffset blockOffset)
 
 			pLine->removeRun(pRun);
 
-			if (pRun->getNext())
-				pRun->getNext()->setPrev(pRun->getPrev());
-
-			if (pRun->getPrev())
-				pRun->getPrev()->setNext(pRun->getNext());
-
 			if (m_pFirstRun == pRun)
 				m_pFirstRun = pRun->getNext();
 
+			pRun->unlinkFromRunList();
+
 			delete pRun;
 
 			if (!m_pFirstRun)
@@ -3377,6 +3626,8 @@ UT_Bool fl_BlockLayout::_deleteFmtMark(PT_BlockOffset blockOffset)
 			
 			// I don't believe that we need to keep looping at this point.
 			// We should not ever have two adjacent FmtMarks....
+			UT_ASSERT(!pNextRun || pNextRun->getType() != FPRUN_FMTMARK);
+
 			break;
 		}
 
@@ -3402,17 +3653,16 @@ UT_Bool fl_BlockLayout::doclistener_changeFmtMark(const PX_ChangeRecord_FmtMarkC
 		if (pRun->getBlockOffset() == blockOffset)
 		{
 			UT_ASSERT(pRun->getType() == FPRUN_FMTMARK);
-			//fp_FieldRun* pFieldRun = static_cast<fp_FieldRun*>(pRun);
-			//pFieldRun->clearScreen();
-			//pFieldRun->lookupProperties();
-			goto done;
+			pRun->lookupProperties();
+			pRun->clearScreen();
+			break;
 		}
 
 		pRun = pRun->getNext();
 	}
 
-done:
-	// TODO is it necessary to force a reformat when changing a FmtMark
+	// We need a reformat for blocks that only contain a format mark.
+	// ie. no next just a carrige return.
 	setNeedsReformat();
 
 	FV_View* pView = m_pLayout->getView();
@@ -3429,10 +3679,10 @@ done:
 void fl_BlockLayout::recheckIgnoredWords()
 {
 	fp_Run	   *pRun = m_pFirstRun;
-	UT_uint32  runBlockOffset = pRun->getBlockOffset();
-	UT_uint32  runLength = pRun->getLength();
+	UT_ASSERT(pRun);
+
 	fl_PartOfBlock*	pPOB;
-	
+
 	// for scanning a word	
 	UT_uint32 wordBeginning, wordLength;
 	UT_Bool bAllUpperCase;
@@ -3491,6 +3741,7 @@ void fl_BlockLayout::recheckIgnoredWords()
 		{
 				// squiggle it
 			pPOB->bIsIgnored = pDoc->isIgnore( &(pBlockText[wordBeginning]), wordLength);
+			_updateSquiggle(pPOB);
 		}
 		else
 		{
@@ -3503,12 +3754,11 @@ void fl_BlockLayout::recheckIgnoredWords()
 			delete pPOB;
 		} // if valid word
 
-		_updateSquiggle(pPOB);
-
 		bUpdate = UT_TRUE;
 	}
 	if (bUpdate && pView)
 	{
+	        pView->_eraseInsertionPoint();
 		pView->updateScreen();
 		pView->_drawInsertionPoint();
 	}
