@@ -42,7 +42,9 @@
 
 #define REDRAW_UPDATE_MSECS	500
 
-FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG) : m_hashFontCache(19)
+FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG)
+:	m_hashFontCache(19),
+	m_pageSize(_getDefaultPageSize())
 {
 	m_pDoc = doc;
 	m_pG = pG;
@@ -65,15 +67,16 @@ FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG) : m_hashFontCache(
 		m_pRedrawUpdateTimer->start();
 	}
 
-
 	// TODO the following (both the new() and the addListener() cause
 	// TODO malloc's to occur.  we are currently inside a constructor
-	// TODO and cannot report failure.
+	// TODO and are not allowed to report failure.
 	
 	m_pDocListener = new fl_DocListener(doc, this);
 	doc->addListener(static_cast<PL_Listener *>(m_pDocListener),&m_lid);
 
-		
+#ifdef FMT_TEST
+	m_pDocLayout = this;
+#endif
 }
 
 FL_DocLayout::~FL_DocLayout()
@@ -323,10 +326,10 @@ fp_Page* FL_DocLayout::addNewPage(fl_DocSectionLayout* pOwner)
 		pLastPage = NULL;
 	}
 
-	/*
-	  TODO the following page dimensions should NOT be hard-coded
-	*/
-	fp_Page*		pPage = new fp_Page(this, m_pView, 850, 1100, pOwner);
+	fp_Page* pPage = new fp_Page(	this,
+									m_pView,
+									m_pageSize,
+									pOwner);
 	if (pLastPage)
 	{
 		UT_ASSERT(pLastPage->getNext() == NULL);
@@ -438,6 +441,13 @@ void FL_DocLayout::updateLayout()
 }
 
 #ifdef FMT_TEST
+FL_DocLayout* FL_DocLayout::m_pDocLayout = NULL;
+
+void __dump(void)
+{
+	FL_DocLayout::m_pDocLayout->__dump(stdout);
+}
+
 void FL_DocLayout::__dump(FILE * fp) const
 {
 	int count = m_vecPages.getItemCount();
@@ -462,6 +472,7 @@ void FL_DocLayout::__dump(FILE * fp) const
 
 void FL_DocLayout::_toggleAutoSpell(UT_Bool bSpell)
 {
+	UT_Bool bOldAutoSpell = m_bAutoSpellCheck;
 	m_bAutoSpellCheck = bSpell;
 
 	UT_DEBUGMSG(("FL_DocLayout::_toggleAutoSpell (%s)\n", bSpell ? "UT_TRUE" : "UT_FALSE" ));
@@ -513,6 +524,12 @@ void FL_DocLayout::_toggleAutoSpell(UT_Bool bSpell)
 			}
 			pSL = (fl_DocSectionLayout *) pSL->getNext();
 		}
+		if (bOldAutoSpell)
+		{
+			// If we're here, it was set to TRUE before but now it is being set
+			// to FALSE. This means that it is the user setting it. That's good.
+			m_pView->draw(NULL);
+		}
 	}
 }
 
@@ -892,3 +909,11 @@ void FL_DocLayout::_redrawUpdate(UT_Timer * pTimer)
 */
 }
 
+fp_PageSize FL_DocLayout::_getDefaultPageSize()
+{
+	// TODO The following page dimensions should NOT be hard-coded.
+	// TODO return PageSize initialized by prefs.
+
+	return fp_PageSize(fp_PageSize::Letter);
+}
+
