@@ -158,7 +158,7 @@ void fp_Line::remove(void)
 
 void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
 {
-	int count = m_vecRuns.getItemCount();
+	const int count = m_vecRuns.getItemCount();
 	UT_ASSERT(count > 0);
 
 	fp_Run* pFirstRun = (fp_Run*) m_vecRuns.getNthItem(0);
@@ -171,7 +171,10 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 
 		UT_sint32 y2 = y - pFirstRun->getY() - m_iAscent + pFirstRun->getAscent();
 		pFirstRun->mapXYToPosition(0, y2, pos, bBOL, bEOL);
-	
+
+		UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+		UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 		return;
 	}
 
@@ -194,6 +197,9 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 				{
 					pRun2->mapXYToPosition(x - pRun2->getX(), y2, pos, bBOL, bEOL);
 
+					UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+					UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 					return;
 				}
 			}
@@ -216,6 +222,9 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 
 					pRun2->mapXYToPosition(x - pRun2->getX(), y2, pos, bBOL, bEOL);
 
+					UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+					UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 					return;
 				}
 			}
@@ -258,10 +267,16 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 	
 	UT_sint32 y2 = y - pClosestRun->getY() - m_iAscent + pClosestRun->getAscent();
 	pClosestRun->mapXYToPosition(x - pClosestRun->getX(), y2, pos, bBOL, bEOL);
+
+	UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+	UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 }
 
 void fp_Line::getOffsets(fp_Run* pRun, UT_sint32& xoff, UT_sint32& yoff)
 {
+  // This returns the baseline of run. ie the bottom of the line of text
+  //
 	UT_sint32 my_xoff;
 	UT_sint32 my_yoff;
 
@@ -411,7 +426,6 @@ void fp_Line::clearScreen(void)
 			m_pContainer->getScreenOffsets(this, xoffLine, yoffLine);
 
 			UT_sint32 columnGap = getColumnGap();
-	
 			pRun->getGraphics()->clearArea(xoffLine - columnGap / 2, yoffLine, m_iMaxWidth + columnGap, m_iHeight);
 		}
 	}
@@ -454,7 +468,7 @@ void fp_Line::clearScreenFromRunToEnd(UT_uint32 runIndex)
 		if(runIndex == 0)
 		{
 			// Erasing the whole line so include 1/2 column gap at start and end.
-			pRun->getGraphics()->clearArea(xoff - getColumnGap() / 2, yoff, m_iMaxWidth + getColumnGap() - (xoff - xoffLine), m_iHeight);
+		        pRun->getGraphics()->clearArea(xoff - getColumnGap() / 2, yoff, m_iMaxWidth + getColumnGap() - (xoff - xoffLine), m_iHeight);
 		}
 		else
 		{
@@ -538,10 +552,11 @@ void fp_Line::layout(void)
 
 	UT_uint32 iCountRuns = m_vecRuns.getItemCount();
 	UT_sint32 iX = 0;
+	UT_sint32 iStartX;
 	UT_uint32 i;
 	UT_Bool bLineErased = UT_FALSE;
 
-	iX = pAlignment->getStartPosition();
+	iStartX = iX = pAlignment->getStartPosition();
 	
 	for (i=0; i<iCountRuns; i++)		// TODO do we need to do this if iMoveOver is zero ??
 	{
@@ -563,7 +578,7 @@ void fp_Line::layout(void)
 			UT_sint32 iPos;
 			unsigned char iTabType;
 
-			UT_Bool bRes = findNextTabStop(iX, iPos, iTabType);
+			UT_Bool bRes = findNextTabStop(iX - iStartX, iPos, iTabType);
 			UT_ASSERT(bRes);
 
 			fp_TabRun* pTabRun = static_cast<fp_TabRun*>(pRun);
@@ -574,8 +589,8 @@ void fp_Line::layout(void)
 			switch ( iTabType )
 			{
 			case FL_TAB_LEFT:
-				pTabRun->setWidth(iPos - iX);
-				iX = iPos;
+				pTabRun->setWidth(iPos - (iX - iStartX));
+				iX = iPos + iStartX;
 				break;
 
 			case FL_TAB_CENTER:
@@ -586,11 +601,11 @@ void fp_Line::layout(void)
 					iScanWidth += pScanRun->getWidth();
 				}
 	
-				if ( iScanWidth / 2 > iPos - iX )
+				if ( iScanWidth / 2 > iPos - (iX - iStartX) )
 					pTabRun->setWidth(0);
 				else
 				{
-					int tabWidth = iPos - iX - iScanWidth / 2 ;
+					int tabWidth = iPos - (iX - iStartX) - iScanWidth / 2 ;
 					pTabRun->setWidth( tabWidth );
 					iX += tabWidth;
 				}
@@ -605,11 +620,11 @@ void fp_Line::layout(void)
 					iScanWidth += pScanRun->getWidth();
 				}
 		
-				if ( iScanWidth > iPos - iX )
+				if ( iScanWidth > iPos - (iX - iStartX) )
 					pTabRun->setWidth(0);
 				else
 				{
-					int tabWidth = iPos - iX - iScanWidth ;
+					int tabWidth = iPos - (iX - iStartX) - iScanWidth ;
 					pTabRun->setWidth( tabWidth );
 					iX += tabWidth;
 				}
@@ -667,9 +682,9 @@ void fp_Line::layout(void)
 				}
 			
 				UT_DEBUGMSG((" tabrun iX=%d iPos=%d iScanWidth=%d tabwidth=%d newX=%d\n", 
-									  iX,	iPos,   iScanWidth,   iPos-iX-iScanWidth,iPos-iScanWidth));	
-				pTabRun->setWidth(iPos - iX - iScanWidth);
-				iX = iPos - iScanWidth;
+									  iX,	iPos,   iScanWidth,   iPos-(iX-iStartX)-iScanWidth,iPos-iScanWidth));	
+				pTabRun->setWidth(iPos - (iX - iStartX) - iScanWidth);
+				iX = iPos - iScanWidth + iStartX;
 			
 				FREEP(pDecimalStr);	
 				break;
@@ -934,28 +949,30 @@ fp_Line*	fp_Line::getPrevLineInSection(void) const
 
 UT_Bool	fp_Line::containsForcedColumnBreak(void) const
 {
-	fp_Run* pRun = getLastRun();
-	if (pRun->getType() == FPRUN_FORCEDCOLUMNBREAK)
+	if(!isEmpty())
 	{
-		return UT_TRUE;
-	}
-	else
-	{
-		return UT_FALSE;
+		fp_Run* pRun = getLastRun();
+		if (pRun->getType() == FPRUN_FORCEDCOLUMNBREAK)
+		{			
+			return UT_TRUE;
+		}
 	}
+
+	return UT_FALSE;
 }
 
 UT_Bool fp_Line::containsForcedPageBreak(void) const
 {
-	fp_Run* pRun = getLastRun();
-	if (pRun->getType() == FPRUN_FORCEDPAGEBREAK)
+	if (!isEmpty())
 	{
-		return UT_TRUE;
-	}
-	else
-	{
-		return UT_FALSE;
+		fp_Run* pRun = getLastRun();
+		if (pRun->getType() == FPRUN_FORCEDPAGEBREAK)
+		{
+			return UT_TRUE;
+		}
 	}
+
+	return UT_FALSE;
 }
 
 void fp_Line::coalesceRuns(void)
@@ -979,12 +996,11 @@ void fp_Line::coalesceRuns(void)
 
 UT_sint32 fp_Line::calculateWidthOfLine(void)
 {
-	UT_uint32 iCountRuns = m_vecRuns.getItemCount();
+	const UT_uint32 iCountRuns = m_vecRuns.getItemCount();
 	UT_sint32 iX = 0;
-	UT_uint32 i;
 
 	// first calc the width of the line
-	for (i=0; i<iCountRuns; i++)
+	for (UT_uint32 i = 0; i < iCountRuns; ++i)
 	{
 		fp_Run* pRun = (fp_Run*) m_vecRuns.getNthItem(i);
 		
@@ -1008,6 +1024,8 @@ UT_sint32 fp_Line::calculateWidthOfLine(void)
 		}
 	}
 
+	UT_ASSERT(iX <= m_iMaxWidth);
+
 	m_iWidth = iX;
 
 	return iX;
@@ -1054,6 +1072,8 @@ UT_sint32 fp_Line::calculateWidthOfTrailingSpaces(void)
 	// need to move back until we find the first non blank character and
 	// return the distance back to this character.
 
+	UT_ASSERT(!isEmpty());
+
 	UT_sint32 iTrailingBlank = 0;
 
 	fp_Run *pCurrentRun = getLastRun();
@@ -1086,6 +1106,8 @@ UT_sint32 fp_Line::calculateWidthOfTrailingSpacesInLayoutUnits(void)
 	// need to move back until we find the first non blank character and
 	// return the distance back to this character.
 
+	UT_ASSERT(!isEmpty());
+
 	UT_sint32 iTrailingBlank = 0;
 
 	fp_Run *pCurrentRun = getLastRun();
@@ -1162,6 +1184,8 @@ UT_uint32 fp_Line::countJustificationPoints(void) const
 
 UT_Bool fp_Line::isLastCharacter(UT_UCSChar Character) const
 {
+	UT_ASSERT(!isEmpty());
+
 	fp_Run *pRun = getLastRun();
 
 	if (pRun->getType() == FPRUN_TEXT)
@@ -1273,3 +1297,4 @@ void fp_Line::splitRunsAtSpaces(void)
 
 }
 
+
