@@ -44,6 +44,19 @@
 // TODO to remember the attr/prop for each run rather than
 // TODO looking it up each time we call lookupProperties() -- jeff 4/19/99
 
+
+// findPointCoords:
+//  Can be called to find the position and size of the point (cursor)
+//  for an offset both before, inside and after the Run.
+//   Before: When the Run is the first on a Line and point is a BOL.
+//   Inside: When the point is inside the Run.
+//   After : Point is at the start of the next Run, but insertion is
+//           done with the properties of this Run so cursor size/position
+//           must reflect that.
+//
+//  Previous implementations would assert that the offset was within the
+//  Run, but that would always fail for the 'After' case.
+
 /*****************************************************************/
 
 /*
@@ -93,8 +106,9 @@ void fp_Run::__dump(FILE * fp) const
 	UT_ASSERT(NrElements(s_names)==(FPRUN__LAST__-FPRUN__FIRST__+1));
 	const char * szName = (((m_iType >= FPRUN__FIRST__) && (m_iType <= FPRUN__LAST__)) ? s_names[m_iType-1] : "Unknown");
 
-	fprintf(fp,"    Run: 0x%p T=%s Off=%d Len=%d D=%c [x %d y %d w %d h %d]\n",
-			this, szName, m_iOffsetFirst, m_iLen, ((m_bDirty) ? 'y' : 'n'),
+	fprintf(fp,"    Run: %p T=%s Off=%d Len=%d D=%c Line=%p [x %d y %d w %d h %d]\n",
+			this, szName, m_iOffsetFirst, m_iLen, 
+			((m_bDirty) ? 'y' : 'n'), m_pLine,
 			m_iX, m_iY, m_iWidth, m_iHeight);
 }
 #endif
@@ -102,6 +116,44 @@ void fp_Run::__dump(FILE * fp) const
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
+void fp_Run::insertIntoRunListBeforeThis(fp_Run& newRun)
+{
+	newRun.unlinkFromRunList();
+	newRun.m_pNext = this;
+	if (m_pPrev)
+	{
+		m_pPrev->m_pNext = &newRun;
+	}
+	newRun.m_pPrev = m_pPrev;
+	m_pPrev = &newRun;
+}
+
+void fp_Run::insertIntoRunListAfterThis(fp_Run& newRun)
+{
+	newRun.unlinkFromRunList();
+	newRun.m_pPrev = this;
+	if (m_pNext)
+	{
+		m_pNext->m_pPrev = &newRun;
+	}
+	newRun.m_pNext = m_pNext;
+	m_pNext = &newRun;
+}
+
+void fp_Run::unlinkFromRunList()
+{
+	if (m_pPrev)
+	{
+		m_pPrev->m_pNext = m_pNext;
+	}
+	if (m_pNext)
+	{
+		m_pNext->m_pPrev = m_pPrev;
+		m_pNext = 0;
+	}
+	m_pPrev = 0;
+}
+
 void	fp_Run::setX(UT_sint32 iX)
 {
 	if (iX == m_iX)
@@ -237,18 +289,18 @@ UT_Bool fp_Run::canContainPoint(void) const
 
 UT_uint32 fp_Run::containsOffset(UT_uint32 iOffset)
 {
-	if (iOffset == (m_iOffsetFirst + m_iLen))
-	{
-		return FP_RUN_JUSTAFTER;
-	}
-	else if ((iOffset >= m_iOffsetFirst) && (iOffset < (m_iOffsetFirst + m_iLen)))
+	if ((iOffset >= m_iOffsetFirst) 
+	    && (iOffset < (m_iOffsetFirst + m_iLen)))
 	{
 		return FP_RUN_INSIDE;
 	}
-	else
-	{
-		return FP_RUN_NOT;
-	}
+
+	return FP_RUN_NOT;
+}
+
+UT_Bool fp_Run::letPointPass(void) const
+{
+	return UT_TRUE;
 }
 
 void fp_Run::fetchCharWidths(fl_CharWidths * /* pgbCharWidths */)
@@ -271,7 +323,24 @@ const PP_AttrProp* fp_Run::getAP(void) const
 	return pSpanAP;
 }
 
+void fp_Run::_drawTextLine(UT_sint32 xoff,UT_sint32 yoff,UT_uint32 iWidth,UT_uint32 iHeight,UT_UCSChar *pText)
+{
+    m_pG->setFont(m_pG->getGUIFont());
 
+    UT_uint32 iTextLen = UT_UCS_strlen(pText);
+    UT_uint32 iTextWidth = m_pG->measureString(pText,0,iTextLen,NULL); 
+    UT_uint32 iTextHeight = m_pG->getFontHeight();
+
+    UT_uint32 xoffText = xoff + (iWidth - iTextWidth) / 2;
+    UT_uint32 yoffText = yoff - m_pG->getFontAscent() * 2 / 3;
+
+    m_pG->drawLine(xoff,yoff,xoff + iWidth,yoff);
+    UT_RGBColor clrPaper(255,255,255);
+    if((iTextWidth < iWidth) && (iTextHeight < iHeight)){
+        m_pG->fillRect(clrPaper,xoffText,yoffText,iTextWidth,iTextHeight);
+        m_pG->drawChars(pText,0,iTextLen,xoffText,yoffText);
+    }
+}
 
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
@@ -294,6 +363,8 @@ void fp_TabRun::lookupProperties(void)
 	FL_DocLayout * pLayout = m_pBL->getDocLayout();
 	GR_Font* pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_SCREEN_RESOLUTION);
 
+    UT_parseColor(PP_evalProperty("color",pSpanAP,pBlockAP,pSectionAP, m_pBL->getDocument(), UT_TRUE), m_colorFG);
+
 	m_pG->setFont(pFont);
 	m_iAscent = m_pG->getFontAscent();	
 	m_iDescent = m_pG->getFontDescent();
@@ -317,21 +388,31 @@ UT_Bool fp_TabRun::canBreakBefore(void) const
 	return UT_FALSE;
 }
 
+UT_Bool fp_TabRun::letPointPass(void) const
+{
+	return UT_TRUE;
+}
+
 UT_Bool	fp_TabRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	return UT_FALSE;
 }
 
-void fp_TabRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /* y */, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
+void fp_TabRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
 {
-	pos = m_pBL->getPosition() + m_iOffsetFirst;
+	// If X is left of the middle, return offset to the left,
+	// otherwise the offset to the right.
+	if (x < (getWidth() / 2))
+		pos = m_pBL->getPosition() + m_iOffsetFirst;
+	else
+		pos = m_pBL->getPosition() + m_iOffsetFirst + m_iLen;
+		
 	bBOL = UT_FALSE;
 	bEOL = UT_FALSE;
 }
 
 void fp_TabRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
 	UT_sint32 xoff;
 	UT_sint32 yoff;
 
@@ -372,6 +453,51 @@ void fp_TabRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 	m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
 }
 
+void fp_TabRun::_drawArrow(UT_uint32 iLeft,UT_uint32 iTop,UT_uint32 iWidth, UT_uint32 iHeight)
+{
+    if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
+        return;
+    }
+
+    #define NPOINTS 8
+
+    UT_Point * points = (UT_Point *)calloc(NPOINTS,sizeof(UT_Point));
+    UT_ASSERT(points);
+
+    UT_sint32 cur_linewidth = 1 + (UT_MAX(10,m_iAscent) - 10) / 8;
+    UT_uint32 iyAxis = iTop + m_pLine->getAscent() * 2 / 3;
+    UT_uint32 iMaxWidth = iWidth / 10 * 6;
+    UT_uint32 ixGap = (iWidth - iMaxWidth) / 2;
+
+    points[0].x = iLeft + ixGap;
+    points[0].y = iyAxis - cur_linewidth / 2;
+
+    points[1].x = iLeft + ixGap + iMaxWidth - cur_linewidth * 4;
+    points[1].y = points[0].y;
+
+    points[2].x = points[1].x;
+    points[2].y = points[0].y - cur_linewidth * 2;
+
+    points[3].x = iLeft + iWidth - ixGap;
+    points[3].y = iyAxis;
+
+    points[4].x = points[1].x;
+    points[4].y = iyAxis + cur_linewidth * 2;
+
+    points[5].x = points[1].x;
+    points[5].y = iyAxis + cur_linewidth / 2;
+
+    points[6].x = points[0].x;
+    points[6].y = points[5].y;
+
+    points[7].x = points[0].x;
+    points[7].y = points[0].y;
+
+    m_pG->setColor(m_colorFG);
+    m_pG->polyLine(points,NPOINTS);
+    FREEP(points);
+}
+
 void fp_TabRun::_draw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
@@ -394,15 +520,22 @@ void fp_TabRun::_draw(dg_DrawArgs* pDA)
 	UT_uint32 iRunBase = m_pBL->getPosition() + m_iOffsetFirst;
 
 	if (
+		pView->getFocus()!=AV_FOCUS_NONE &&
 		(iSel1 <= iRunBase)
 		&& (iSel2 > iRunBase)
 		)
 	{
 		m_pG->fillRect(clrSelBackground, pDA->xoff, iFillTop, m_iWidth, iFillHeight);
+        if(pView->getShowPara()){
+            _drawArrow(pDA->xoff, iFillTop, m_iWidth, iFillHeight);
+        }
 	}
 	else
 	{
 		m_pG->fillRect(clrNormalBackground, pDA->xoff, iFillTop, m_iWidth, iFillHeight);
+        if(pView->getShowPara()){
+            _drawArrow(pDA->xoff, iFillTop, m_iWidth, iFillHeight);
+        }
 	}
 }
 
@@ -443,6 +576,11 @@ UT_Bool fp_ForcedLineBreakRun::canBreakBefore(void) const
 	return UT_FALSE;
 }
 
+UT_Bool fp_ForcedLineBreakRun::letPointPass(void) const
+{
+	return UT_FALSE;
+}
+
 UT_Bool	fp_ForcedLineBreakRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -462,17 +600,6 @@ void fp_ForcedLineBreakRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/,
 void fp_ForcedLineBreakRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
-	UT_sint32 xoff;
-	UT_sint32 yoff;
-
-	UT_ASSERT(m_pLine);
-	
-	m_pLine->getOffsets(this, xoff, yoff);
-	x = xoff;
-	y = yoff;
-	height = m_pLine->getHeight();
 }
 
 void fp_ForcedLineBreakRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
@@ -546,6 +673,11 @@ UT_Bool fp_ImageRun::canBreakBefore(void) const
 	return UT_TRUE;
 }
 
+UT_Bool fp_ImageRun::letPointPass(void) const
+{
+	return UT_FALSE;
+}
+
 UT_Bool	fp_ImageRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	return UT_FALSE;
@@ -554,22 +686,16 @@ UT_Bool	fp_ImageRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeft
 void fp_ImageRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
 {
 	if (x > m_iWidth)
-	{
 		pos = m_pBL->getPosition() + m_iOffsetFirst + m_iLen;
-		bBOL = UT_FALSE;
-		bEOL = UT_FALSE;
-	}
 	else
-	{
 		pos = m_pBL->getPosition() + m_iOffsetFirst;
-		bBOL = UT_FALSE;
-		bEOL = UT_FALSE;
-	}
+
+	bBOL = UT_FALSE;
+	bEOL = UT_FALSE;
 }
 
 void fp_ImageRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
 	UT_sint32 xoff;
 	UT_sint32 yoff;
 
@@ -610,9 +736,16 @@ void fp_ImageRun::_draw(dg_DrawArgs* pDA)
 
 	yoff += m_pLine->getAscent() - m_iAscent;
 	
+	// clip drawing to the page
+	UT_Rect pClipRect;
+	pClipRect.top = yoff;
+	pClipRect.left = xoff;
+	pClipRect.height = m_pLine->getContainer()->getHeight();
+	pClipRect.width = m_pLine->getContainer()->getWidth();
+	m_pG->setClipRect(&pClipRect);
+
 	if (m_pImage)
 	{
-
 		// draw the image (always)
 		m_pG->drawImage(m_pImage, xoff, yoff);
 
@@ -631,6 +764,7 @@ void fp_ImageRun::_draw(dg_DrawArgs* pDA)
 			UT_ASSERT(iSel1 <= iSel2);
 	
 			if (
+				pView->getFocus()!=AV_FOCUS_NONE &&
 				(iSel1 <= iRunBase)
 				&& (iSel2 > iRunBase)
 				)
@@ -663,6 +797,9 @@ void fp_ImageRun::_draw(dg_DrawArgs* pDA)
 		UT_RGBColor clr(0, 0, 255);
 		m_pG->fillRect(clr, xoff, yoff, m_iWidth, m_iHeight);
 	}
+
+	// unf*ck clipping rect
+	m_pG->setClipRect(NULL);
 }
 
 //////////////////////////////////////////////////////////////////
@@ -824,6 +961,8 @@ void fp_FieldRun::lookupProperties(void)
 	const PP_AttrProp * pSectionAP = NULL; // TODO do we care about section-level inheritance?
 	
 	m_pBL->getSpanAttrProp(m_iOffsetFirst,UT_FALSE,&pSpanAP);
+	
+	PD_Document * pDoc = m_pBL->getDocument();
 	m_pBL->getAttrProp(&pBlockAP);
 
 	// look for fonts in this DocLayout's font cache
@@ -834,6 +973,7 @@ void fp_FieldRun::lookupProperties(void)
 	UT_parseColor(PP_evalProperty("color",pSpanAP,pBlockAP,pSectionAP, m_pBL->getDocument(), UT_TRUE), m_colorFG);
 
 	m_pG->setFont(m_pFont);
+
 	m_iAscent = m_pG->getFontAscent();	
 	m_iDescent = m_pG->getFontDescent();
 	m_iHeight = m_pG->getFontHeight();
@@ -846,6 +986,19 @@ void fp_FieldRun::lookupProperties(void)
 	m_pG->setFont(m_pFont);
 
 	const XML_Char* pszType = NULL;
+
+	const XML_Char * pszPosition = PP_evalProperty("text-position",pSpanAP,pBlockAP,pSectionAP, pDoc, UT_TRUE);
+
+	if (0 == UT_stricmp(pszPosition, "superscript"))
+	{
+		m_fPosition = TEXT_POSITION_SUPERSCRIPT;
+	}
+	else if (0 == UT_stricmp(pszPosition, "subscript"))
+	{
+		m_fPosition = TEXT_POSITION_SUBSCRIPT;
+	} 
+	else m_fPosition = TEXT_POSITION_NORMAL;
+
 	pSpanAP->getAttribute("type", pszType);
 	UT_ASSERT(pszType);
 
@@ -874,28 +1027,63 @@ UT_Bool fp_FieldRun::canBreakBefore(void) const
 	return UT_TRUE;
 }
 
-UT_Bool	fp_FieldRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
+UT_Bool fp_FieldRun::letPointPass(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool        fp_FieldRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* 
+iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	return UT_FALSE;
 }
 
-void fp_FieldRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
+UT_Bool fp_FieldRun::isSuperscript(void) const 
 {
-	pos = m_pBL->getPosition() + m_iOffsetFirst;
+	return (m_fPosition == TEXT_POSITION_SUPERSCRIPT);
+}
+
+UT_Bool fp_FieldRun::isSubscript(void) const
+{
+	return (m_fPosition == TEXT_POSITION_SUBSCRIPT);
+}
+
+void fp_FieldRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
+{
+	// If X is left of the middle, return offset to the left,
+	// otherwise the offset to the right.
+	if (x < (getWidth() / 2))
+		pos = m_pBL->getPosition() + m_iOffsetFirst;
+	else
+		pos = m_pBL->getPosition() + m_iOffsetFirst + m_iLen;
+		
 	bBOL = UT_FALSE;
 	bEOL = UT_FALSE;
 }
 
 void fp_FieldRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
 	UT_sint32 xoff;
 	UT_sint32 yoff;
 
 	UT_ASSERT(m_pLine);
 	
 	m_pLine->getOffsets(this, xoff, yoff);
-	x = xoff;
+
+	if (iOffset == (m_iOffsetFirst + m_iLen))
+	{
+		xoff += m_iWidth;
+	}
+	if (m_fPosition == TEXT_POSITION_SUPERSCRIPT)
+	{
+		yoff -= m_iAscent * 1/2;
+	}
+	else if (m_fPosition == TEXT_POSITION_SUBSCRIPT)
+	{
+		yoff += m_iDescent /* * 3/2 */;
+	}
+
+        x = xoff;
 	y = yoff;
 	height = m_iHeight;
 }
@@ -917,6 +1105,19 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
 
+
+
+	UT_sint32 iYdraw =  pDA->yoff - getAscent()-1;
+	
+	if (m_fPosition == TEXT_POSITION_SUPERSCRIPT)
+	{
+	        iYdraw -= getAscent() * 1/2;
+	}
+        else if (m_fPosition == TEXT_POSITION_SUBSCRIPT) 
+	{
+	        iYdraw +=  getDescent(); // * 3/2   
+	} 
+ 
 	if (m_pG->queryProperties(GR_Graphics::DGP_SCREEN))
 	{
 		UT_uint32 iRunBase = m_pBL->getPosition() + m_iOffsetFirst;
@@ -929,12 +1130,11 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 		  with fields always being drawn a little darker than the
 		  surrounding text.
 		*/
-		
 		UT_RGBColor clrSelBackground(112, 112, 112);
 		UT_RGBColor clrNormalBackground(220, 220, 220);
-
-		UT_sint32 iFillHeight = m_pLine->getHeight();
-		UT_sint32 iFillTop = pDA->yoff - m_pLine->getAscent();
+		
+		UT_sint32 iFillTop = iYdraw;
+		UT_sint32 iFillHeight = getAscent() + getDescent();
 		
 		FV_View* pView = m_pBL->getDocLayout()->getView();
 		UT_uint32 iSelAnchor = pView->getSelectionAnchor();
@@ -946,6 +1146,7 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 		UT_ASSERT(iSel1 <= iSel2);
 	
 		if (
+			pView->getFocus()!=AV_FOCUS_NONE &&
 			(iSel1 <= iRunBase)
 			&& (iSel2 > iRunBase)
 			)
@@ -961,7 +1162,7 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 	m_pG->setFont(m_pFont);
 	m_pG->setColor(m_colorFG);
 	
-	m_pG->drawChars(m_sFieldValue, 0, UT_UCS_strlen(m_sFieldValue), pDA->xoff, pDA->yoff - m_iAscent);
+	m_pG->drawChars(m_sFieldValue, 0, UT_UCS_strlen(m_sFieldValue), pDA->xoff,iYdraw);
 }
 
 //////////////////////////////////////////////////////////////////
@@ -999,6 +1200,11 @@ UT_Bool fp_ForcedColumnBreakRun::canBreakBefore(void) const
 	return UT_FALSE;
 }
 
+UT_Bool fp_ForcedColumnBreakRun::letPointPass(void) const
+{
+	return UT_FALSE;
+}
+
 UT_Bool	fp_ForcedColumnBreakRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -1015,28 +1221,40 @@ void fp_ForcedColumnBreakRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/
 
 void fp_ForcedColumnBreakRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
-	UT_sint32 xoff;
-	UT_sint32 yoff;
-
-	UT_ASSERT(m_pLine);
-	
-	m_pLine->getOffsets(this, xoff, yoff);
-	x = xoff;
-	y = yoff;
-	height = m_pLine->getHeight();
-
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 }
 
 void fp_ForcedColumnBreakRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 {
 	UT_ASSERT(!m_bDirty);
 	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
+
+    UT_sint32 xoff = 0, yoff = 0;
+    m_pLine->getScreenOffsets(this, xoff, yoff);
+    UT_sint32 iWidth  = m_pLine->getMaxWidth() - m_pLine->calculateWidthOfLine();
+    m_pG->clearArea(xoff,yoff,iWidth,m_pLine->getHeight());
 }
 
 void fp_ForcedColumnBreakRun::_draw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
+
+    if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
+        return;
+    }
+
+    FV_View* pView = m_pBL->getDocLayout()->getView();
+    UT_ASSERT(pView);
+    if(!pView->getShowPara()){
+        return;
+    }
+
+    UT_sint32 iLineWidth  = m_pLine->getMaxWidth() - m_pLine->calculateWidthOfLine();
+
+    UT_UCSChar *pColumnBreak;
+    UT_UCS_cloneString_char(&pColumnBreak,"Column Break");
+    _drawTextLine(pDA->xoff,pDA->yoff - m_pLine->getAscent() / 3,iLineWidth,m_pLine->getHeight(),pColumnBreak);
+    FREEP(pColumnBreak);
 }
 
 //////////////////////////////////////////////////////////////////
@@ -1071,6 +1289,11 @@ UT_Bool fp_ForcedPageBreakRun::canBreakBefore(void) const
 	return UT_FALSE;
 }
 
+UT_Bool fp_ForcedPageBreakRun::letPointPass(void) const
+{
+	return UT_FALSE;
+}
+
 UT_Bool	fp_ForcedPageBreakRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
 {
 	return UT_FALSE;
@@ -1085,26 +1308,39 @@ void fp_ForcedPageBreakRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/,
 
 void fp_ForcedPageBreakRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
-	UT_sint32 xoff;
-	UT_sint32 yoff;
-
-	UT_ASSERT(m_pLine);
-	
-	m_pLine->getOffsets(this, xoff, yoff);
-	x = xoff;
-	y = yoff;
-	height = m_pLine->getHeight();
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 }
 
 void fp_ForcedPageBreakRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 {
 	UT_ASSERT(!m_bDirty);
 	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
+
+    UT_sint32 xoff = 0, yoff = 0;
+    m_pLine->getScreenOffsets(this, xoff, yoff);
+    UT_sint32 iWidth  = m_pLine->getMaxWidth() - m_pLine->calculateWidthOfLine();
+    m_pG->clearArea(xoff,yoff,iWidth,m_pLine->getHeight());
 }
 
 void fp_ForcedPageBreakRun::_draw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
+
+    if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
+        return;
+    }
+
+    FV_View* pView = m_pBL->getDocLayout()->getView();
+    UT_ASSERT(pView);
+    if(!pView->getShowPara()){
+        return;
+    }
+
+    UT_sint32 iLineWidth  = m_pLine->getMaxWidth() - m_pLine->calculateWidthOfLine();
+
+    UT_UCSChar *pPageBreak;
+    UT_UCS_cloneString_char(&pPageBreak,"Page Break");
+    _drawTextLine(pDA->xoff,pDA->yoff - m_pLine->getAscent() / 3,iLineWidth,m_pLine->getHeight(),pPageBreak);
+    FREEP(pPageBreak);
 }
 
