@@ -44,7 +44,8 @@ fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
 					   UT_uint32 iOffsetFirst,
 					   UT_uint32 iLen,
 					   UT_Bool bLookupProperties)
-	: fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_TEXT)
+:	fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_TEXT),
+	m_fPosition(TEXT_POSITION_NORMAL)
 {
 	m_pFont = NULL;
 	m_pFontLayout = NULL;
@@ -250,12 +251,8 @@ UT_Bool fp_TextRun::alwaysFits(void) const
 
 		return UT_FALSE;
 	}
-	else
-	{
-		// could assert here -- this should never happen, I think
-		return UT_TRUE;
-	}
 
+	// could assert here -- this should never happen, I think
 	return UT_TRUE;
 }
 
@@ -338,17 +335,24 @@ UT_Bool	fp_TextRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidt
 
 void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
 {
-	if  (x <= 0)
+	if (x <= 0)
 	{
 		pos = m_pBL->getPosition() + m_iOffsetFirst;
 		// don't set bBOL to false here
 		bEOL = UT_FALSE;
+
+		UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 		return;
 	}
 
 	if (x >= m_iWidth)
 	{
 		pos = m_pBL->getPosition() + m_iOffsetFirst + m_iLen;
+
+		UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+		UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 		return;
 	}
 
@@ -379,6 +383,10 @@ void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& p
 			bEOL = UT_TRUE;
 
 			pos = m_pBL->getPosition() + i;
+
+			UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
+			UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
+
 			return;
 		}
 	}
@@ -388,8 +396,6 @@ void fp_TextRun::mapXYToPosition(UT_sint32 x, UT_sint32 /*y*/, PT_DocPosition& p
 
 void fp_TextRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
 {
-//	UT_ASSERT(FP_RUN_NOT != containsOffset(iOffset));
-	
 	UT_sint32 xoff;
 	UT_sint32 yoff;
 
@@ -421,23 +427,16 @@ void fp_TextRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y,
 
 UT_Bool fp_TextRun::canMergeWithNext(void)
 {
-	if (
-		(!m_pNext)
-		|| (m_pNext->getType() != FPRUN_TEXT)
-		)
+	if (!m_pNext ||
+		!m_pLine ||
+		m_pNext->getType() != FPRUN_TEXT ||
+		!m_pNext->getLine())
 	{
 		return UT_FALSE;
 	}
 
-	if (
-		!(m_pLine)
-	    || !(m_pNext->getLine())
-		)
-	{
-		return UT_FALSE;
-	}
 	
-	fp_TextRun* pNext = (fp_TextRun*) m_pNext;
+	fp_TextRun* pNext = static_cast<fp_TextRun*>(m_pNext);
 	if (
 		(pNext->m_iOffsetFirst != (m_iOffsetFirst + m_iLen))
 		|| (pNext->m_fDecorations != m_fDecorations)
@@ -499,6 +498,7 @@ UT_Bool fp_TextRun::split(UT_uint32 iSplitOffset)
 	pNew->m_pFontLayout = this->m_pFontLayout;
 	pNew->m_fDecorations = this->m_fDecorations;
 	pNew->m_colorFG = this->m_colorFG;
+        pNew->m_fPosition = this->m_fPosition;
 
 	pNew->m_iAscent = this->m_iAscent;
 	pNew->m_iDescent = this->m_iDescent;
@@ -707,7 +707,7 @@ void fp_TextRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 		}
 		else
 		{
-			m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
+		        m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
 		}
 	}
 
@@ -720,8 +720,8 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	  the top.
 	*/
 
-	UT_sint32 yTopOfRun = pDA->yoff - m_iAscent;
-	UT_sint32 yTopOfSel = yTopOfRun;
+        UT_sint32 yTopOfRun = pDA->yoff - m_iAscent-1; // Hack to remove
+	UT_sint32 yTopOfSel = yTopOfRun+1; // final character dirt
 
 	if (m_fPosition == TEXT_POSITION_SUPERSCRIPT)
 	{
@@ -757,7 +757,7 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	
 	UT_ASSERT(iSel1 <= iSel2);
 	
-	if (iSel1 == iSel2)
+	if (pView->getFocus()==AV_FOCUS_NONE || iSel1 == iSel2)
 	{
 		// nothing in this run is selected
 		_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, m_iLen, pgbCharWidths);
@@ -810,6 +810,9 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	}
 
 	_drawDecors(pDA->xoff, yTopOfRun);
+    if(pView->getShowPara()){
+        _drawInvisibles(pDA->xoff, yTopOfRun);
+    }
 
 	// TODO: draw this underneath (ie, before) the text and decorations
 	m_bSquiggled = UT_FALSE;
@@ -962,20 +965,22 @@ class. If a underline or overline is pending (because the next run continues
 drawn.
      */
 
-  if( (m_fDecorations & (TEXT_DECOR_UNDERLINE | TEXT_DECOR_OVERLINE | 
-			TEXT_DECOR_LINETHROUGH)) == 0) return;
-        UT_sint32 old_LineWidth = m_iLineWidth;
-        UT_sint32 cur_linewidth = 1+ (UT_MAX(10,m_iAscent)-10)/8;
+	if( (m_fDecorations & (TEXT_DECOR_UNDERLINE | TEXT_DECOR_OVERLINE | 
+			TEXT_DECOR_LINETHROUGH)) == 0)
+	{
+		return;
+	}
+
+	const UT_sint32 old_LineWidth = m_iLineWidth;
+	UT_sint32 cur_linewidth = 1+ (UT_MAX(10,m_iAscent)-10)/8;
 	UT_sint32 iDrop = 0;
-        fp_Run* P_Run = getPrev();
-        fp_Run* N_Run = getNext();
-        UT_Bool b_Underline = isUnderline();
-        UT_Bool b_Overline = isOverline();
-	UT_Bool b_Strikethrough = isStrikethrough();
-	const UT_UCSChar* pSpan;
-	UT_uint32 lenSpan;
-	UT_Bool b_Firstrun = (P_Run == NULL) || (m_pLine->getFirstRun()== this);
-	UT_Bool b_Lastrun = (N_Run == NULL) || (m_pLine->getLastRun()== this);
+	fp_Run* P_Run = getPrev();
+	fp_Run* N_Run = getNext();
+	const UT_Bool b_Underline = isUnderline();
+	const UT_Bool b_Overline = isOverline();
+	const UT_Bool b_Strikethrough = isStrikethrough();
+	const UT_Bool b_Firstrun = (P_Run == NULL) || (m_pLine->getFirstRun()== this);
+	const UT_Bool b_Lastrun = (N_Run == NULL) || (m_pLine->getLastRun()== this);
 
 	/*
 	  If the previous run is NULL or if this is the first run of a line, 
@@ -1116,6 +1121,56 @@ text so we can keep the original code.
 	m_pG->setLineWidth(m_iLineWidth);
 }
 
+void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
+{
+    UT_GrowBuf * pgbCharWidths = m_pBL->getCharWidths()->getCharWidths();
+    UT_uint16* pCharWidths = pgbCharWidths->getPointer(0);
+    const UT_UCSChar* pSpan;
+    UT_uint32 lenSpan;
+    UT_uint32 len = m_iLen;
+    UT_sint32 iWidth = 0;
+    UT_sint32 cur_linewidth = 1+ (UT_MAX(10,m_iAscent)-10)/8;
+    UT_sint32 iRectSize = cur_linewidth * 3 / 2;
+    UT_Bool bContinue = UT_TRUE;
+    UT_uint32 offset = m_iOffsetFirst;
+
+    if(findCharacter(0, UCS_SPACE) > 0){
+        while(bContinue){
+            bContinue = m_pBL->getSpanPtr(offset,&pSpan,&lenSpan);
+            UT_ASSERT(lenSpan > 0);
+
+
+            if(lenSpan > len){
+                lenSpan = len;
+            }
+
+            UT_uint32 iy = yoff + getAscent() * 2 / 3;
+
+            for (UT_uint32 i = 0;i < lenSpan;i++){
+               if(pSpan[i] == UCS_SPACE){
+                   m_pG->fillRect(m_colorFG,xoff + iWidth + (pCharWidths[i + offset] - iRectSize) / 2,iy,iRectSize,iRectSize);
+               }
+               iWidth += pCharWidths[i + offset];
+            }
+            if (len <= lenSpan){
+                bContinue = UT_FALSE;
+            }else{
+                offset += lenSpan;
+                len -= lenSpan;
+            }
+
+        }
+    }
+}
+
+void fp_TextRun::_drawInvisibles(UT_sint32 xoff, UT_sint32 yoff)
+{
+    if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
+        return;
+    }
+    _drawInvisibleSpaces(xoff,yoff);
+}
+
 void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 {
 	if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN)))
