@@ -202,7 +202,8 @@ UT_Error PD_Document::save(void)
 
 	// make sure we don't cause extension problems
 
-	if(strcmp(UT_pathSuffix(m_szFilename), ".doc") == 0)
+	const char* szSuffix = UT_pathSuffix(m_szFilename);
+	if(szSuffix && strcmp(szSuffix, ".doc") == 0)
 	{
 	  UT_DEBUGMSG(("PD_Document::Save -- word extensions cause problems\n"));
 	  return UT_EXTENSIONERROR;
@@ -254,16 +255,23 @@ UT_Bool	PD_Document::insertObject(PT_DocPosition dpos,
 
 UT_Bool PD_Document::insertSpan(PT_DocPosition dpos,
 								  const UT_UCSChar * p,
-								  UT_uint32 length)
+								  UT_uint32 length,
+								  PP_AttrProp *p_AttrProp)
 {
+	if(p_AttrProp)
+	{
+		m_pPieceTable->insertFmtMark(PTC_AddFmt, dpos, p_AttrProp);
+	}
+
 	return m_pPieceTable->insertSpan(dpos,p,length);
 }
 
 UT_Bool PD_Document::deleteSpan(PT_DocPosition dpos1,
-								PT_DocPosition dpos2)
+								PT_DocPosition dpos2,
+								PP_AttrProp *p_AttrProp_Before)
 {
-	return m_pPieceTable->deleteSpan(dpos1,dpos2);
-}
+	return m_pPieceTable->deleteSpan(dpos1, dpos2, p_AttrProp_Before);
+}												 
 
 UT_Bool PD_Document::changeSpanFmt(PTChangeFmt ptc,
 								   PT_DocPosition dpos1,
