@@ -34,6 +34,12 @@ public:
 
 	virtual void			runModal(XAP_Frame * pFrame);
 
+	virtual void	                runModeless(XAP_Frame * pFrame){};
+        virtual void                    destroy(void){};
+        virtual void                    activate(void){};
+	virtual void	 notifyActiveFrame(XAP_Frame *pFrame) {};
+	virtual void	 notifyCloseFrame(XAP_Frame *pFrame){};
+
 	static XAP_Dialog *		static_constructor(XAP_DialogFactory *, XAP_Dialog_Id id);
 	
 protected:
