@@ -44,6 +44,9 @@
 #include "ap_Strings.h"
 #include "xap_EditMethods.h"
 #include "ap_LoadBindings.h"
+#include "xap_DialogFactory.h"
+#include "xap_Dlg_MessageBox.h"
+#include "xap_Dialog_Id.h"
 #include "xap_Menu_ActionSet.h"
 #include "xap_Toolbar_ActionSet.h"
 #include "xav_View.h"
@@ -346,6 +349,7 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, UT_Bool bUseCl
 
 	static const char * aszFormatsAccepted[] = { AP_CLIPBOARD_RTF,
 												 AP_CLIPBOARD_TEXTPLAIN_8BIT,
+												 AP_CLIPBOARD_STRING,
 												 0 /* must be last */ };
 
 	// TODO currently i have this set so that a ^v or Menu[Edit/Paste] will
@@ -383,7 +387,8 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, UT_Bool bUseCl
 		return;
 	}
 
-	if (UT_stricmp(szFormatFound,AP_CLIPBOARD_TEXTPLAIN_8BIT) == 0)
+	if (   (UT_stricmp(szFormatFound,AP_CLIPBOARD_TEXTPLAIN_8BIT) == 0)
+	    || (UT_stricmp(szFormatFound,AP_CLIPBOARD_STRING) == 0))
 	{
 		iLen = MyMin(iLen,strlen((const char *)pData));
 		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in format [%s].\n",iLen,szFormatFound));
@@ -465,13 +470,24 @@ void AP_UnixApp::setSelectionStatus(AV_View * pView)
 		m_pClipboard->clearData(UT_FALSE,UT_TRUE);
 	}
 	
-	m_pViewSelection = pView;
+	UT_DEBUGMSG(("here we go whooooo\n"));
+	setViewSelection(pView);
 	m_pFrameSelection = (XAP_Frame *)pView->getParentData();
 
 	m_bSelectionInFlux = UT_FALSE;
 	return;
 }
 
+void    AP_UnixApp::setViewSelection( AV_View * pView)
+{
+        m_pViewSelection = pView;
+}
+
+AV_View* AP_UnixApp::getViewSelection(void)
+{
+        return m_pViewSelection;
+}
+
 UT_Bool AP_UnixApp::forgetFrame(XAP_Frame * pFrame)
 {
 	// we intercept this so that we can erase our
@@ -480,10 +496,13 @@ UT_Bool AP_UnixApp::forgetFrame(XAP_Frame * pFrame)
 	// view pointer of a closed window when the
 	// selection is changed....
 
+
 	if (m_pFrameSelection && (pFrame==m_pFrameSelection))
 	{
 		m_pClipboard->clearData(UT_FALSE,UT_TRUE);
 		m_pFrameSelection = NULL;
+		UT_DEBUGMSG(("here we go wheeeee\n"));
+		
 		m_pViewSelection = NULL;
 	}
 	
@@ -512,6 +531,7 @@ void AP_UnixApp::clearSelection(void)
 	
 	if (m_pViewSelection && m_pFrameSelection && m_bHasSelection)
 	{
+		UT_DEBUGMSG(("crash2\n"));
 		m_pViewSelection->cmdUnselectSelection();
 		m_bHasSelection = UT_FALSE;
 	}
@@ -974,6 +994,64 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 					// TODO if the user gave us 2 bogus pathnames....
 					kWindowsOpened++;
 					pFirstUnixFrame->loadDocument(NULL, IEFT_Unknown);
+
+					pFirstUnixFrame->raise();
+					
+					XAP_DialogFactory * pDialogFactory
+						= (XAP_DialogFactory *)(pFirstUnixFrame->getDialogFactory());
+					
+					XAP_Dialog_MessageBox * pDialog
+						= (XAP_Dialog_MessageBox *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_MESSAGE_BOX));
+					UT_ASSERT(pDialog);
+					
+					const XAP_StringSet * pSS = pFirstUnixFrame->getApp()->getStringSet();
+					
+					switch (error)
+					{
+					case -301:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_FileNotFound),m_pArgs->m_argv[k]);
+						break;
+						
+					case -302:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_NoMemory),m_pArgs->m_argv[k]);
+						break;
+						
+					case -303:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_UnknownType),m_pArgs->m_argv[k]);
+						break;
+						
+					case -304:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_BogusDocument),m_pArgs->m_argv[k]);
+						break;
+						
+					case -305:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotOpen),m_pArgs->m_argv[k]);
+						break;
+							
+					case -306:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotWrite),m_pArgs->m_argv[k]);
+						break;
+						
+					case -307:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_FakeType),m_pArgs->m_argv[k]);
+						break;
+						
+					case -311:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_UnsupportedType),m_pArgs->m_argv[k]);
+						break;
+						
+					default:
+						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_ImportError),m_pArgs->m_argv[k]);
+					}
+					pDialog->setButtons(XAP_Dialog_MessageBox::b_O);
+					pDialog->setDefaultAnswer(XAP_Dialog_MessageBox::a_OK);
+					
+					pDialog->runModal(pFirstUnixFrame);
+					
+					XAP_Dialog_MessageBox::tAnswer ans = pDialog->getAnswer();
+					
+					pDialogFactory->releaseDialog(pDialog);
+					
 #else
 					delete pFirstUnixFrame;
 #endif
