@@ -44,6 +44,7 @@
 #include "xap_UnixFontManager.h"
 #include "ap_UnixStatusBar.h"
 #include "ap_UnixViewListener.h"
+#include"ut_dialogHelper.h"
 
 #ifdef ABISOURCE_LICENSED_TRADEMARKS
 #include "abiword_48_tm.xpm"
@@ -92,7 +93,7 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 	AV_ListenerId lidScrollbarViewListener;
 	UT_uint32 nrToolbars;
 	UT_uint32 point = 0;
-
+	gboolean bFocus;
 	XAP_UnixFontManager * fontManager = ((XAP_UnixApp *) getApp())->getFontManager();
 	
 	pG = new GR_UnixGraphics(m_dArea->window, fontManager);
@@ -110,7 +111,8 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 		point = ((FV_View *) m_pView)->getPoint();
 	}
 	ENSUREP(pView);
-
+	bFocus=GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(m_wTopLevelWindow),"toplevelWindowFocus"));
+	pView->setFocus(bFocus && (gtk_grab_get_current()==NULL || gtk_grab_get_current()==m_wTopLevelWindow) ? AV_FOCUS_HERE : !bFocus && gtk_grab_get_current()!=NULL && isTransientWindow(GTK_WINDOW(gtk_grab_get_current()),GTK_WINDOW(m_wTopLevelWindow)) ?  AV_FOCUS_NEARBY : AV_FOCUS_NONE);
 	// The "AV_ScrollObj pScrollObj" receives
 	// send{Vertical,Horizontal}ScrollEvents
 	// from both the scroll-related edit methods
@@ -189,6 +191,8 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 		UNREFP(pOldDoc);
 	}
 	REPLACEP(m_pView, pView);
+        if(getApp()->getViewSelection())
+	       getApp()->setViewSelection(pView);
 	REPLACEP(m_pScrollObj, pScrollObj);
 	REPLACEP(m_pViewListener, pViewListener);
 	m_lid = lid;
@@ -212,6 +216,7 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 	((AP_FrameData*)m_pData)->m_pStatusBar->setView(pView);
 
 	pView->setInsertMode(((AP_FrameData*)m_pData)->m_bInsertMode);
+    ((FV_View *) m_pView)->setShowPara(((AP_FrameData*)m_pData)->m_bShowPara);
 	
 	m_pView->setWindowSize(GTK_WIDGET(m_dArea)->allocation.width,
 						   GTK_WIDGET(m_dArea)->allocation.height);
