@@ -29,24 +29,22 @@
 
 class XAP_Frame;
 
-class AP_Dialog_WordCount : public XAP_Dialog_NonPersistent
+class AP_Dialog_WordCount : public XAP_Dialog_Modeless
 {
 public:
 	AP_Dialog_WordCount(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id);
 	virtual ~AP_Dialog_WordCount(void);
 
-	virtual void					runModal(XAP_Frame * pFrame) = 0;
-
 	typedef enum { a_OK, a_CANCEL } tAnswer;
 
 	AP_Dialog_WordCount::tAnswer		getAnswer(void) const;
-	FV_DocCount                             getCount(void) const;
-	void                                    setCount(FV_DocCount);
-	
+	FV_DocCount							getCount(void) const;
+	void								setCount(FV_DocCount);
+	void								setCountFromActiveFrame(void);
+
 protected:
-	
 	AP_Dialog_WordCount::tAnswer		m_answer;
-	FV_DocCount                             m_count;
+	FV_DocCount							m_count;
 };
 
 #endif /* AP_DIALOG_WORDCOUNT_H */
