@@ -535,6 +535,11 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 		break;
 
 	case AP_MENU_ID_VIEW_SHOWPARA:
+        if ( pFrameData->m_bShowPara )
+            s = EV_MIS_Toggled;
+        else
+            s = EV_MIS_ZERO;
+        break;
 	case AP_MENU_ID_VIEW_HEADFOOT:
 		// TODO: implement view methods to check, toggle state
 		s = EV_MIS_Gray;
