@@ -197,7 +197,14 @@ UT_Bool AP_TopRuler::notify(AV_View * pView, const AV_ChangeMask mask)
 	// indents), then we redraw the ruler.
 	
 	if (mask & (AV_CHG_COLUMN | AV_CHG_FMTSECTION | AV_CHG_FMTBLOCK))
-		draw(NULL);
+	{
+		UT_Rect pClipRect;
+		pClipRect.top = 0;
+		pClipRect.left = UT_MAX(m_iLeftRulerWidth, s_iFixedWidth);
+		pClipRect.height = m_iHeight;
+		pClipRect.width = m_iWidth;
+		draw(&pClipRect);
+	}
 	
 	return UT_TRUE;
 }
@@ -1210,8 +1217,11 @@ void AP_TopRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 
 void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButton /* emb */, UT_sint32 x, UT_sint32 y)
 {
-	if (!m_bValidMouseClick)
+	if (!m_bValidMouseClick || (m_bEventIgnored && m_draggingWhat != DW_TABSTOP))
+	{
+		m_draggingWhat = DW_NOTHING;
 		return;
+	}
 
 	m_bValidMouseClick = UT_FALSE;
 
@@ -1224,18 +1234,8 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 		return;
 	}
 
-	// if they drag horizontally off the ruler, we probably want to ignore
-	// the whole thing.  but this may interact with the current scroll.
-
-	UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
-	if ((x < xFixed) || (x > (UT_sint32)m_iWidth))
-	{
-		_ignoreEvent(UT_TRUE);
-		m_draggingWhat = DW_NOTHING;
-		return;
-	}
-
-	// mouse up was in the ruler portion of the window, we cannot ignore it.
+	// mouse up was in the ruler portion of the window, or horizontally
+	// off - we cannot ignore it.
 	// i'd like to assert that we can just use the data computed in the
 	// last mouseMotion() since the Release must be at the same spot or
 	// we'd have received another Motion before the release.  therefore,
@@ -1279,13 +1279,14 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 
 			_xorGuide(UT_TRUE);
 			m_draggingWhat = DW_NOTHING;
-			(static_cast<FV_View *>(m_pView))->setSectionFormat(properties);
+            FV_View *pView = static_cast<FV_View *>(m_pView);
+            pView->setSectionFormat(properties);
+            pView->draw(NULL);
 		}
 		return;
 
 	case DW_RIGHTMARGIN:
 		{
-			UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
 			UT_sint32 xAbsRight = _getFirstPixelInColumn(&m_infoCache, m_infoCache.m_iNumColumns - 1) + 
 								m_infoCache.u.c.m_xColumnWidth + m_infoCache.u.c.m_xaRightMargin;
 			double dxrel = _scalePixelDistanceToUnits(xAbsRight - m_draggingCenter, tick);
@@ -1298,7 +1299,9 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 
 			_xorGuide(UT_TRUE);
 			m_draggingWhat = DW_NOTHING;
-			(static_cast<FV_View *>(m_pView))->setSectionFormat(properties);
+            FV_View *pView = static_cast<FV_View *>(m_pView);
+            pView->setSectionFormat(properties);
+            pView->draw(NULL);
 		}
 		return;
 
@@ -1314,7 +1317,9 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 			UT_DEBUGMSG(("TopRuler: ColumnGap [%s]\n",properties[1]));
 
 			m_draggingWhat = DW_NOTHING;
-			(static_cast<FV_View *>(m_pView))->setSectionFormat(properties);
+            FV_View *pView = static_cast<FV_View *>(m_pView);
+            pView->setSectionFormat(properties);
+            pView->draw(NULL);
 		}
 		return;
 		
@@ -1408,13 +1413,12 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 			unsigned char iType;
 			UT_sint32 iTab = _findTabStop(&m_infoCache, x, y, iType);
 
-			if ((iTab >= 0) && (iTab == m_draggingTab))
+			if (iTab >= 0)
 			{
-				// this tabstop is already set here ==> NOOP
+				// a tabstop is already set here ==> delete the one we're dragging
 				m_draggingWhat = DW_NOTHING;
-				return;
+				_setTabStops(tick, iTab, UT_TRUE); // UT_TRUE for the last arg will cause this to be deleted
 			}
-
 			_setTabStops(tick, iTab, UT_FALSE);
 		}
 		return;
@@ -1484,6 +1488,8 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 	if (!m_bValidMouseClick)
 		return;
 		
+	m_bEventIgnored = UT_FALSE;
+
 	UT_DEBUGMSG(("mouseMotion: [ems 0x%08lx][x %ld][y %ld]\n",ems,x,y));
 
 	// if they drag vertically off the ruler, we ignore the whole thing.
@@ -1502,21 +1508,21 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 	// the whole thing.  but this may interact with the current scroll.
 
 	UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
-	if ((x < xFixed) || (x > (UT_sint32)m_iWidth))
+	UT_sint32 xStartPixel = xFixed + (UT_sint32) m_infoCache.m_xPageViewMargin;
+	UT_sint32 xAbsRight = _getFirstPixelInColumn(&m_infoCache, m_infoCache.m_iNumColumns - 1) + 
+		m_infoCache.u.c.m_xColumnWidth + m_infoCache.u.c.m_xaRightMargin;
+	ap_RulerTicks tick(m_pG,m_dim);
+
+	if (x < xStartPixel)
 	{
-		if(!m_bEventIgnored)
-		{
-			_ignoreEvent(UT_FALSE);
-			m_bEventIgnored = UT_TRUE;
-		}
-		return;
+		x = xStartPixel;
+	}
+	else if (x > xAbsRight)
+	{
+		x = xAbsRight;
 	}
 
-	m_bEventIgnored = UT_FALSE;
-
 	// mouse motion was in the ruler portion of the window, we cannot ignore it.
-	
-	ap_RulerTicks tick(m_pG,m_dim);
 
 	switch (m_draggingWhat)
 	{
@@ -1526,29 +1532,23 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 		
 	case DW_LEFTMARGIN:
 		{
-		double xPosition = _getUnitsFromRulerLeft(x, tick);
-
 		UT_sint32 oldDragCenter = m_draggingCenter;
 
-		UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
 		UT_sint32 xAbsLeft = xFixed + m_infoCache.m_xPageViewMargin - m_xScrollOffset;
 
 		m_draggingCenter = _snapPixelToGrid(x, tick);
-		if(m_draggingCenter < xAbsLeft)
+
+        UT_sint32 iAbsLeft = _getFirstPixelInColumn(&m_infoCache,0);
+        UT_sint32 iAbsRight = iAbsLeft + m_infoCache.u.c.m_xColumnWidth;
+        UT_sint32 iIndentShift = UT_MAX(0,UT_MAX(m_infoCache.m_xrLeftIndent,m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent));
+        UT_sint32 iRightIndentPos = iAbsRight - UT_MAX(0,m_infoCache.m_xrRightIndent) - iIndentShift;
+        if(iRightIndentPos - m_draggingCenter < m_minColumnWidth)
 		{
-			// don't drag off the page.
-			m_draggingCenter = xAbsLeft;
-		}
-		if(m_draggingCenter + m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent < xAbsLeft)
-		{
-			// don't drag first line indent off page.
-			m_draggingCenter = xAbsLeft - m_infoCache.m_xrLeftIndent - m_infoCache.m_xrFirstLineIndent;
-		}
-		if(m_draggingCenter + m_infoCache.m_xrLeftIndent < xAbsLeft)
-		{
-			// don't drag left indent off page.
-			m_draggingCenter = xAbsLeft - m_infoCache.m_xrLeftIndent;
+            m_draggingCenter = iRightIndentPos - m_minColumnWidth;
 		}
+        iIndentShift = UT_MIN(0,UT_MIN(m_infoCache.m_xrLeftIndent,m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent));
+        m_draggingCenter = UT_MAX(m_draggingCenter, xAbsLeft - iIndentShift);
+
 		if(m_draggingCenter == oldDragCenter)
 		{
 			// Position not changing so finish here.
@@ -1599,16 +1599,28 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 	case DW_RIGHTMARGIN:
 		{
 		UT_sint32 oldDragCenter = m_draggingCenter;
+        UT_sint32 iAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
+        UT_sint32 iRightShift = UT_MAX(0,m_infoCache.m_xrRightIndent);
+        UT_sint32 iLeftShift = UT_MAX(0,UT_MAX(m_infoCache.m_xrLeftIndent,m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent));
+        UT_sint32 newMargin;
+        UT_sint32 deltaRightMargin;
+        UT_sint32 newColumnWidth;
 
-		UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
-		UT_sint32 xAbsRight = _getFirstPixelInColumn(&m_infoCache, m_infoCache.m_iNumColumns - 1) + 
-								m_infoCache.u.c.m_xColumnWidth + m_infoCache.u.c.m_xaRightMargin;
+        x = UT_MIN(x,xAbsRight + UT_MIN(0,m_infoCache.m_xrRightIndent));
+        while(1){
+            newMargin = xAbsRight - x;
+            deltaRightMargin = newMargin - m_infoCache.u.c.m_xaRightMargin;
+            newColumnWidth = m_infoCache.u.c.m_xColumnWidth - deltaRightMargin / (UT_sint32)m_infoCache.m_iNumColumns;
 
-		m_draggingCenter = _snapPixelToGrid(x, tick);
-		if(m_draggingCenter > xAbsRight)
-		{
-			m_draggingCenter = xAbsRight;
+            if(newColumnWidth - iRightShift - iLeftShift < m_minColumnWidth){
+                x += (m_minColumnWidth - (newColumnWidth - iRightShift - iLeftShift)) * (UT_sint32)m_infoCache.m_iNumColumns;
+            } else {
+                break;
 		}
+        }
+
+        m_draggingCenter = _snapPixelToGrid(x,tick);
+
 		if(m_draggingCenter == oldDragCenter)
 		{
 			// Position not changing so finish here.
@@ -1616,18 +1628,6 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			return;
 		}
 
-		UT_sint32 newMargin = xAbsRight - m_draggingCenter;
-		UT_sint32 deltaRightMargin = newMargin - m_infoCache.u.c.m_xaRightMargin;
-		UT_sint32 newColumnWidth = m_infoCache.u.c.m_xColumnWidth - deltaRightMargin / (UT_sint32)m_infoCache.m_iNumColumns;
-		if(newColumnWidth < m_minColumnWidth)
-		{
-			x += (m_minColumnWidth - newColumnWidth) * (UT_sint32)m_infoCache.m_iNumColumns;
-
-			m_draggingCenter = _snapPixelToGrid(x, tick);
-			newMargin = xAbsRight - m_draggingCenter;
-			deltaRightMargin = newMargin - m_infoCache.u.c.m_xaRightMargin;
-			
-		}
 		m_infoCache.u.c.m_xaRightMargin += deltaRightMargin;
 		m_infoCache.u.c.m_xColumnWidth -= deltaRightMargin / (UT_sint32)m_infoCache.m_iNumColumns;
 
@@ -1649,9 +1649,9 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 	case DW_COLUMNGAPLEFTSIDE:
 		{
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,0);
-			UT_sint32 xAbsRight = xAbsLeft + m_infoCache.u.c.m_xColumnWidth;
-			UT_sint32 xAbsRightGap = xAbsRight + m_infoCache.u.c.m_xColumnGap;
-			UT_sint32 xAbsMidPoint = (xAbsRight + xAbsRightGap)/2;
+			UT_sint32 xAbsRight2 = xAbsLeft + m_infoCache.u.c.m_xColumnWidth;
+			UT_sint32 xAbsRightGap = xAbsRight2 + m_infoCache.u.c.m_xColumnGap;
+			UT_sint32 xAbsMidPoint = (xAbsRight2 + xAbsRightGap)/2;
 			UT_sint32 xrel;
 			
 			if(m_draggingWhat == DW_COLUMNGAP)
@@ -1659,7 +1659,7 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 				UT_sint32 xAbsLowerLimit = xAbsMidPoint + _snapPixelToGrid((UT_sint32)(tick.dragDelta/tick.tickUnitScale),tick);
 				if (((UT_sint32)x) < xAbsLowerLimit)
 					x = (UT_uint32)xAbsLowerLimit;
-				xrel = ((UT_sint32)x) - xAbsRight;
+				xrel = ((UT_sint32)x) - xAbsRight2;
 			}
 			else
 			{
@@ -1683,11 +1683,11 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
 			UT_Rect oldDraggingRect = m_draggingRect;
-			m_draggingCenter = xAbsRight + xgrid;
+			m_draggingCenter = xAbsRight2 + xgrid;
 			_getColumnMarkerRect(&m_infoCache,0,m_draggingCenter,&m_draggingRect);
 
 
-			UT_DEBUGMSG(("Gap: [x %ld][xAbsRight %ld][xrel %ld][xgrid %ld][width %ld]\n",x,xAbsRight,xrel,xgrid,m_draggingRect.width));
+			UT_DEBUGMSG(("Gap: [x %ld][xAbsRight %ld][xrel %ld][xgrid %ld][width %ld]\n",x,xAbsRight2,xrel,xgrid,m_draggingRect.width));
 
 
 			if (!m_bBeforeFirstMotion && (m_draggingCenter != oldDraggingCenter))
@@ -1711,8 +1711,6 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			// to update it (since it is stored in relative to the
 			// paragraph in the document).
 			
-			// TODO what upper/lower bound should we place on this ?
-			
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
 			UT_sint32 xrel = ((UT_sint32)x) - xAbsLeft;
 			UT_sint32 xgrid = _snapPixelToGrid(xrel,tick);
@@ -1721,6 +1719,13 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
 			UT_Rect oldDraggingRect = m_draggingRect;
 			m_draggingCenter = xAbsLeft + xgrid;
+
+            UT_sint32 iRightPos = m_infoCache.u.c.m_xColumnWidth + xAbsLeft - m_infoCache.m_xrRightIndent;
+            if(iRightPos - m_draggingCenter < m_minColumnWidth)
+            {
+                m_draggingCenter = iRightPos - m_minColumnWidth;
+            }
+
 			_getParagraphMarkerRects(&m_infoCache,m_draggingCenter,0,0,&m_draggingRect,NULL,NULL);
 			if (!m_bBeforeFirstMotion && (m_draggingCenter != oldDraggingCenter))
 				draw(&oldDraggingRect,&m_infoCache);
@@ -1745,8 +1750,6 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			// first-line-indent (since it is stored in relative
 			// terms.
 			
-			// TODO what upper/lower bound should we place on this ?
-			
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
 			UT_sint32 xrel = ((UT_sint32)x) - xAbsLeft;
 			UT_sint32 xgrid = _snapPixelToGrid(xrel,tick);
@@ -1754,10 +1757,29 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			double dgrid = _scalePixelDistanceToUnits(xrel,tick);
 			UT_DEBUGMSG(("SettingLeftIndent: %s\n",m_pG->invertDimension(tick.dimType,dgrid)));
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
+			UT_sint32 oldDragging2Center = m_dragging2Center;
 			UT_Rect oldDraggingRect = m_draggingRect;
 			UT_Rect oldDragging2Rect = m_dragging2Rect;
 			m_draggingCenter  = xAbsLeft + xgrid;
 			m_dragging2Center = xAbsLeft + xgridTagAlong;
+
+            UT_sint32 iFirstIndentShift = UT_MAX(0,m_infoCache.m_xrFirstLineIndent);
+            UT_sint32 iRightIndentPos = xAbsLeft + m_infoCache.u.c.m_xColumnWidth - m_infoCache.m_xrRightIndent - iFirstIndentShift;
+
+			// Prevent the first-line indent from being dragged off the page
+			if (m_dragging2Center < xFixed + m_infoCache.m_xPageViewMargin)
+			{
+				m_dragging2Center = oldDragging2Center;
+				m_draggingCenter = oldDraggingCenter;
+				return;
+			}
+
+            if(iRightIndentPos - m_draggingCenter < m_minColumnWidth)
+            {
+                m_draggingCenter = iRightIndentPos - m_minColumnWidth;
+                m_dragging2Center = m_draggingCenter + xgridTagAlong - xgrid;
+            }
+
 			_getParagraphMarkerRects(&m_infoCache,
 									 m_draggingCenter,0,m_dragging2Center,
 									 &m_draggingRect,NULL,&m_dragging2Rect);
@@ -1778,35 +1800,36 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 		
 	case DW_RIGHTINDENT:
 		{
-			// TODO what upper/lower bound should we place on this ?
-			
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
-			UT_sint32 xAbsRight = xAbsLeft + m_infoCache.u.c.m_xColumnWidth;
-			UT_sint32 xrel = xAbsRight - ((UT_sint32)x);
+			UT_sint32 xAbsRight2 = xAbsLeft + m_infoCache.u.c.m_xColumnWidth;
+			UT_sint32 xrel = xAbsRight2 - ((UT_sint32)x);
 			UT_sint32 xgrid = _snapPixelToGrid(xrel,tick);
 			double dgrid = _scalePixelDistanceToUnits(xrel,tick);
 			UT_DEBUGMSG(("SettingRightIndent: %s\n",m_pG->invertDimension(tick.dimType,dgrid)));
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
 			UT_Rect oldDraggingRect = m_draggingRect;
-			m_draggingCenter = xAbsRight - xgrid;
+			m_draggingCenter = xAbsRight2 - xgrid;
+
+            UT_sint32 iLeftIndentPos = xAbsLeft + UT_MAX(m_infoCache.m_xrLeftIndent,m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent);
+            if(m_draggingCenter - iLeftIndentPos < m_minColumnWidth)
+            {
+                m_draggingCenter = iLeftIndentPos + m_minColumnWidth;
+            }
+
 			_getParagraphMarkerRects(&m_infoCache,0,m_draggingCenter,0,NULL,&m_draggingRect,NULL);
 			if (!m_bBeforeFirstMotion && (m_draggingCenter != oldDraggingCenter))
 				draw(&oldDraggingRect,&m_infoCache);
 			_drawParagraphProperties(NULL,&m_infoCache,UT_FALSE);
 			_xorGuide();
 
-			double dxrel = _scalePixelDistanceToUnits(xAbsRight - m_draggingCenter,tick);
+			double dxrel = _scalePixelDistanceToUnits(xAbsRight2 - m_draggingCenter,tick);
 			_displayStatusMessage(AP_STRING_ID_RightIndentStatus, tick, dxrel);
-
-
 		}
 		m_bBeforeFirstMotion = UT_FALSE;
 		return;
 
 	case DW_FIRSTLINEINDENT:
 		{
-			// TODO what upper/lower bound should we place on this ?
-			
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
 			UT_sint32 xrel = ((UT_sint32)x) - xAbsLeft;
 			// first-line-indent is relative to the left-indent
@@ -1818,6 +1841,13 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
 			UT_Rect oldDraggingRect = m_draggingRect;
 			m_draggingCenter = xAbsLeft + m_infoCache.m_xrLeftIndent + xgrid;
+
+            UT_sint32 iRightIndentPos = xAbsLeft + m_infoCache.u.c.m_xColumnWidth - m_infoCache.m_xrRightIndent;
+            if(iRightIndentPos - m_draggingCenter  < m_minColumnWidth)
+            {
+                m_draggingCenter = iRightIndentPos - m_minColumnWidth;
+            }
+
 			_getParagraphMarkerRects(&m_infoCache,0,0,m_draggingCenter,NULL,NULL,&m_draggingRect);
 			if (!m_bBeforeFirstMotion && (m_draggingCenter != oldDraggingCenter))
 				draw(&oldDraggingRect,&m_infoCache);
@@ -1835,8 +1865,6 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 
 	case DW_TABSTOP:
 		{
-			// TODO what upper/lower bound should we place on this ?
-			
 			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
 			UT_sint32 xrel = ((UT_sint32)x) - xAbsLeft;
 			UT_sint32 xgrid = _snapPixelToGrid(xrel,tick);
