@@ -378,8 +378,8 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 	const PP_AttrProp * pAP = NULL;
 	UT_Bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
-	bool span = UT_FALSE;
-	bool textD = UT_FALSE;
+	UT_Bool span = UT_FALSE;
+	UT_Bool textD = UT_FALSE;
 	
 	if (bHaveProp && pAP)
 	{
@@ -854,7 +854,7 @@ s_HTML_Listener::s_HTML_Listener(PD_Document * pDocument,
 	
 	m_pie->write("\n");
 	
-	m_pie->write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"DTD/xhtml1-strict.dtd\">\n");
+	m_pie->write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml/DTD/xhtml1-strict.dtd\">\n");
 	m_pie->write("<html>\n");
 	m_pie->write("<head>\n");
 	m_pie->write("<title>AbiWord Document</title>\n");
