@@ -38,6 +38,15 @@
 /*****************************************************************/
 /*****************************************************************/
 
+#define DEFAULT_SIZE "12pt"
+
+enum JustificationTypes {
+	JUSTIFIED,
+	CENTER,
+	RIGHT,
+	LEFT
+};
+
 IE_Exp_LaTeX::IE_Exp_LaTeX(PD_Document * pDocument)
 	: IE_Exp(pDocument)
 {
@@ -136,6 +145,8 @@ protected:
 	UT_Bool				m_bInSpan;
 	const PP_AttrProp*	m_pAP_Span;
 	UT_Bool             m_bMultiCols;
+	JustificationTypes  m_eJustification;
+	UT_Bool				m_bLineHeight;
 
 	// Need to look up proper type, and place to stick #defines...
 
@@ -168,6 +179,24 @@ void s_LaTeX_Listener::_closeBlock(void)
 	switch (m_iBlockType)
 	{
 	case BT_NORMAL:
+		switch (m_eJustification)
+		{
+		case JUSTIFIED:
+			break;
+		case CENTER:
+			m_pie->write("\n\\end{center}");
+			break;
+		case RIGHT:
+			m_pie->write("\n\\end{flushright}");
+			break;
+		case LEFT:
+			m_pie->write("\n\\end{flushleft}");
+			break;
+		}
+
+		if (m_bLineHeight)
+		  m_pie->write("\n\\end{spacing}");
+
 		m_pie->write("\n\n");
 		break;
 	case BT_HEADING1:
@@ -191,6 +220,9 @@ void s_LaTeX_Listener::_closeBlock(void)
 
 void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 {
+	m_eJustification = JUSTIFIED;
+	m_bLineHeight = UT_FALSE;
+
 	if (!m_bInSection)
 	{
 		return;
@@ -242,14 +274,42 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 		}
 		
 		/* Assumption: never get property set with h1-h3, block text, plain text. Probably true. */
-		
-		if (
-			m_iBlockType == BT_NORMAL && (pAP->getProperty("text-align", szValue))
-			)
+		// TODO: Split this function in several ones
+		if (m_iBlockType == BT_NORMAL)
 		{
-// 			m_pie->write(" ALIGN=\"");
-// 			m_pie->write(szValue);
-// 			m_pie->write("\"");
+			if (pAP->getProperty("text-align", szValue))
+			{
+				if (0 == UT_stricmp(szValue, "center"))
+				{
+					m_pie->write("\\begin{center}\n");
+					m_eJustification = CENTER;
+				}
+				if (0 == UT_stricmp(szValue, "right"))
+				{
+					m_pie->write("\\begin{flushright}\n");
+					m_eJustification = RIGHT;
+				}
+				if (0 == UT_stricmp(szValue, "left"))
+				{
+					m_pie->write("\\begin{flushleft}\n");
+					m_eJustification = LEFT;
+				}
+			}
+
+			if (pAP->getProperty("line-height", szValue))
+			{
+				if (0 != UT_stricmp(szValue, "1.0"))
+				{
+					m_pie->write("\\begin{spacing}{");
+					m_bLineHeight = UT_TRUE;
+				}
+				if (0 == UT_stricmp(szValue, "1.5"))
+					m_pie->write("1.24}\n");
+				else if (0 == UT_stricmp(szValue, "2.0"))
+					m_pie->write("1.66}\n");
+				else // glup.  TODO: calculate the spacing :)
+				    m_pie->write("1.0} % Sorry.  I know that you don't expected the 1.0... feel free to fix it! :)\n");
+			}
 		}
 	}
 	else 
@@ -269,10 +329,12 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 	if (m_pDocument->getAttrProp(api, &pAP) && pAP)
 	{
 		const XML_Char* pszNbCols = NULL;
-		pAP->getProperty("columns", pszNbCols);
+		const XML_Char* pszPageMarginLeft = NULL;
+		const XML_Char* pszPageMarginRight = NULL;
 
-		if (pszNbCols != NULL)
-			printf ("%s\n", pszNbCols);
+		pAP->getProperty("columns", pszNbCols);
+		pAP->getProperty("page-margin-right", pszPageMarginLeft);
+		pAP->getProperty("page-margin-left", pszPageMarginRight);
 
 		if (pszNbCols != NULL && ((0 == UT_stricmp(pszNbCols, "2"))
 								  || (0 == UT_stricmp(pszNbCols, "3"))))
@@ -282,7 +344,24 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 			m_pie->write("}\n");
 			m_bMultiCols = UT_TRUE;
 		}
+		if (pszPageMarginLeft != NULL)
+		{
+			m_pie->write("\\setlength{\\oddsidemargin}{");
+			m_pie->write(pszPageMarginLeft);
+			m_pie->write("-1in");
+			m_pie->write("}\n");
+		}
+		if (pszPageMarginRight != NULL)
+		{
+			m_pie->write("\\setlength{\\textwidth}{\\paperwidth - ");
+			m_pie->write(pszPageMarginRight);
+			m_pie->write("-");
+			m_pie->write(pszPageMarginLeft);
+			m_pie->write("}\n");
+		}
 	}
+
+	m_pie->write ("\n\n\\begin{document}\n");
 }
 
 void s_LaTeX_Listener::_convertColor(char* szDest, const char* pszColor)
@@ -422,11 +501,14 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 
 		if (pAP->getProperty("font-size", szValue))
 		{
-			m_pie->write("{\\");
-			char szSize[16];
-			_convertFontSize(szSize, szValue);
-			m_pie->write(szSize);
-			m_pie->write("{}");
+			if (strcmp (szValue, DEFAULT_SIZE) != 0)
+			{
+				m_pie->write("{\\");
+				char szSize[16];
+				_convertFontSize(szSize, szValue);
+				m_pie->write(szSize);
+				m_pie->write("{}");
+			}
 		}
 		
 		if (pAP->getProperty("font-family", szValue))
@@ -448,13 +530,13 @@ void s_LaTeX_Listener::_closeSpan(void)
 	{
 		const XML_Char * szValue;
 		
-		if (
-//			(pAP->getProperty("color", szValue)) ||    // TODO
+		if (// (pAP->getProperty("color", szValue)) ||    // TODO
 		    (pAP->getProperty("font-size", szValue))
 //		    || (pAP->getProperty("font-family", szValue))  // TODO
 			)
 		{
-			m_pie->write("}");
+			if (strcmp (szValue, DEFAULT_SIZE) != 0)
+				m_pie->write("}");
 		}
 
 		if (pAP->getProperty("text-position", szValue))
@@ -569,29 +651,19 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 	char * pBuf;
 	const UT_UCSChar * pData;
 
-	for (pBuf=buf, pData=data; (pData<data+length); /**/)
+	for (pBuf = buf, pData = data; (pData < data + length); /**/)
 	{
-		if (pBuf >= (buf+MY_BUFFER_SIZE-MY_HIGHWATER_MARK))
+		if (pBuf >= (buf + MY_BUFFER_SIZE - MY_HIGHWATER_MARK))
 		{
-			m_pie->write(buf,(pBuf-buf));
+			m_pie->write (buf, (pBuf - buf));
 			pBuf = buf;
 		}
 
 		switch (*pData)
 		{
 		case '\\':
-			*pBuf++ = '$';
-			*pBuf++ = '\\';
-			*pBuf++ = 'b';
-			*pBuf++ = 'a';
-			*pBuf++ = 'c';
-			*pBuf++ = 'k';
-			*pBuf++ = 's';
-			*pBuf++ = 'l';
-			*pBuf++ = 'a';
-			*pBuf++ = 's';
-			*pBuf++ = 'h';
-			*pBuf++ = '$';
+			strncpy (pBuf, "\\ensuremath{\\backslash}}", MY_BUFFER_SIZE);
+			pBuf += strlen ("\\ensuremath{\\backslash}}");
 			pData++;
 			break;
 			
@@ -750,10 +822,12 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 	
 	m_pie->write("\\documentclass[12pt]{article}\n");
 	m_pie->write("\\usepackage[T1]{fontenc}\n");
+	m_pie->write("\\usepackage{calc}\n");
+	m_pie->write("\\usepackage{setspace}\n");
 	m_pie->write("\\usepackage{multicol}\t% TODO: I don't need this package if the document is a single column one.\n");
 	m_pie->write("\n");
-	m_pie->write("\\begin{document}\n");
-	m_pie->write("\n");
+	//	m_pie->write("\\begin{document}\n");  // I've to leave this step to the openSection, and that implies
+	//	m_pie->write("\n");                   // future problems when we will support several sections in the same doc...
 }
 
 s_LaTeX_Listener::~s_LaTeX_Listener()
