@@ -217,7 +217,7 @@ void s_RTF_ListenerWriteDoc::_outputData(const UT_UCSChar * data, UT_uint32 leng
 				// unicode sequence.  when i originally did this, i chose
 				// hex, so i'm not going to change it now.
 				
-				m_pie->_rtf_keyword_hex2("'",*pData);
+				m_pie->_rtf_nonascii_hex2(*pData);
 				pData++;
 			}
 			else
@@ -292,8 +292,8 @@ UT_Bool s_RTF_ListenerWriteDoc::populate(PL_StruxFmtHandle /*sfh*/,
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
 			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
-			PT_AttrPropIndex api = pcr->getIndexAP();
-			switch (pcro->getObjectType())
+			//PT_AttrPropIndex api = pcr->getIndexAP();
+		       switch (pcro->getObjectType())
 			{
 			case PTO_Image:
 				_closeSpan();
@@ -581,8 +581,6 @@ void s_RTF_ListenerWriteDoc::_rtf_open_block(PT_AttrPropIndex api)
 		// TODO the following parser was copied from abi/src/text/fmt/xp/fl_BlockLayout.cpp
 		// TODO we should extract both of them and share the code.
 
-		unsigned char iType = 0;
-		UT_sint32 iPosition = 0;
 		UT_Vector vecTabs;
 		
 		const char* pStart = szTabStops;
