@@ -295,14 +295,9 @@ UT_Error IE_Imp_RTF::_parseFile(FILE* fp)
 						cNibble = 2;
 						b = 0;
 						m_currentRTFState.m_internalState = RTFStateStore::risNorm;
-						if (ok)  // skip following space if applicable
-						{   
-							unsigned char ch;
-							if (!ReadCharFromFile(&ch))
-								ok = UT_FALSE; 
-							else if (ch != ' ') 
-								SkipBackChar(ch);
-						}   
+                        // actually don't handle the following space since
+                        // this is NOT a delimiter
+						// see bug #886
 					}
 				}
 			}
@@ -319,7 +314,7 @@ UT_Error IE_Imp_RTF::_parseFile(FILE* fp)
 /*****************************************************************/
 /*****************************************************************/
 
-UT_Bool IE_Imp_RTF::RecognizeContents(const char * szBuf, int iNumbytes)
+UT_Bool IE_Imp_RTF::RecognizeContents(const char * szBuf, UT_uint32 iNumbytes)
 {
 	if ( iNumbytes < 5 )
 	{
@@ -993,7 +988,10 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 		ParseChar(UCS_NBSP);
 		break;
 	case '-':
+		// TODO handle optional hyphen. Currently simply ignore them.
+		break;
 	case '_':
+		// currently simply make a standard hyphen
 		ParseChar('-');	// TODO - make these optional and nonbreaking
 		break;
 	}
